from nebius.api.buf.validate import validate_pb2 as _validate_pb2
from nebius.api.nebius.common.v1 import metadata_pb2 as _metadata_pb2
from nebius.api.nebius.msp.v1alpha1 import cluster_pb2 as _cluster_pb2
from nebius.api.nebius.msp.postgresql.v1alpha1.config import postgresql_pb2 as _postgresql_pb2
from nebius.api.nebius.msp.postgresql.v1alpha1 import template_pb2 as _template_pb2
from nebius.api.nebius.msp.v1alpha1.resource import template_pb2 as _template_pb2_1
from nebius.api.nebius import annotations_pb2 as _annotations_pb2
from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class Cluster(_message.Message):
    __slots__ = ["metadata", "spec", "status"]
    METADATA_FIELD_NUMBER: _ClassVar[int]
    SPEC_FIELD_NUMBER: _ClassVar[int]
    STATUS_FIELD_NUMBER: _ClassVar[int]
    metadata: _metadata_pb2.ResourceMetadata
    spec: ClusterSpec
    status: ClusterStatus
    def __init__(self, metadata: _Optional[_Union[_metadata_pb2.ResourceMetadata, _Mapping]] = ..., spec: _Optional[_Union[ClusterSpec, _Mapping]] = ..., status: _Optional[_Union[ClusterStatus, _Mapping]] = ...) -> None: ...

class ConnectionPoolerConfig(_message.Message):
    __slots__ = ["pooling_mode", "max_pool_size"]
    class PoolingMode(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
        __slots__ = []
        POOLING_MODE_UNSPECIFIED: _ClassVar[ConnectionPoolerConfig.PoolingMode]
        SESSION: _ClassVar[ConnectionPoolerConfig.PoolingMode]
        TRANSACTION: _ClassVar[ConnectionPoolerConfig.PoolingMode]
    POOLING_MODE_UNSPECIFIED: ConnectionPoolerConfig.PoolingMode
    SESSION: ConnectionPoolerConfig.PoolingMode
    TRANSACTION: ConnectionPoolerConfig.PoolingMode
    POOLING_MODE_FIELD_NUMBER: _ClassVar[int]
    MAX_POOL_SIZE_FIELD_NUMBER: _ClassVar[int]
    pooling_mode: ConnectionPoolerConfig.PoolingMode
    max_pool_size: int
    def __init__(self, pooling_mode: _Optional[_Union[ConnectionPoolerConfig.PoolingMode, str]] = ..., max_pool_size: _Optional[int] = ...) -> None: ...

class ClusterSpec(_message.Message):
    __slots__ = ["description", "network_id", "config", "bootstrap", "backup"]
    DESCRIPTION_FIELD_NUMBER: _ClassVar[int]
    NETWORK_ID_FIELD_NUMBER: _ClassVar[int]
    CONFIG_FIELD_NUMBER: _ClassVar[int]
    BOOTSTRAP_FIELD_NUMBER: _ClassVar[int]
    BACKUP_FIELD_NUMBER: _ClassVar[int]
    description: str
    network_id: str
    config: ConfigSpec
    bootstrap: BootstrapSpec
    backup: BackupSpec
    def __init__(self, description: _Optional[str] = ..., network_id: _Optional[str] = ..., config: _Optional[_Union[ConfigSpec, _Mapping]] = ..., bootstrap: _Optional[_Union[BootstrapSpec, _Mapping]] = ..., backup: _Optional[_Union[BackupSpec, _Mapping]] = ...) -> None: ...

class ClusterStatus(_message.Message):
    __slots__ = ["phase", "state", "preset_details", "connection_endpoints"]
    PHASE_FIELD_NUMBER: _ClassVar[int]
    STATE_FIELD_NUMBER: _ClassVar[int]
    PRESET_DETAILS_FIELD_NUMBER: _ClassVar[int]
    CONNECTION_ENDPOINTS_FIELD_NUMBER: _ClassVar[int]
    phase: _cluster_pb2.ClusterStatus.Phase
    state: _cluster_pb2.ClusterStatus.State
    preset_details: _template_pb2_1.PresetDetails
    connection_endpoints: Endpoints
    def __init__(self, phase: _Optional[_Union[_cluster_pb2.ClusterStatus.Phase, str]] = ..., state: _Optional[_Union[_cluster_pb2.ClusterStatus.State, str]] = ..., preset_details: _Optional[_Union[_template_pb2_1.PresetDetails, _Mapping]] = ..., connection_endpoints: _Optional[_Union[Endpoints, _Mapping]] = ...) -> None: ...

class Endpoints(_message.Message):
    __slots__ = ["private_read_write", "private_read_only", "public_read_write", "public_read_only"]
    PRIVATE_READ_WRITE_FIELD_NUMBER: _ClassVar[int]
    PRIVATE_READ_ONLY_FIELD_NUMBER: _ClassVar[int]
    PUBLIC_READ_WRITE_FIELD_NUMBER: _ClassVar[int]
    PUBLIC_READ_ONLY_FIELD_NUMBER: _ClassVar[int]
    private_read_write: str
    private_read_only: str
    public_read_write: str
    public_read_only: str
    def __init__(self, private_read_write: _Optional[str] = ..., private_read_only: _Optional[str] = ..., public_read_write: _Optional[str] = ..., public_read_only: _Optional[str] = ...) -> None: ...

class ConfigSpec(_message.Message):
    __slots__ = ["version", "pooler_config", "postgresql_config_16", "public_access", "template"]
    VERSION_FIELD_NUMBER: _ClassVar[int]
    POOLER_CONFIG_FIELD_NUMBER: _ClassVar[int]
    POSTGRESQL_CONFIG_16_FIELD_NUMBER: _ClassVar[int]
    PUBLIC_ACCESS_FIELD_NUMBER: _ClassVar[int]
    TEMPLATE_FIELD_NUMBER: _ClassVar[int]
    version: str
    pooler_config: ConnectionPoolerConfig
    postgresql_config_16: _postgresql_pb2.PostgresqlConfig16
    public_access: bool
    template: _template_pb2.TemplateSpec
    def __init__(self, version: _Optional[str] = ..., pooler_config: _Optional[_Union[ConnectionPoolerConfig, _Mapping]] = ..., postgresql_config_16: _Optional[_Union[_postgresql_pb2.PostgresqlConfig16, _Mapping]] = ..., public_access: bool = ..., template: _Optional[_Union[_template_pb2.TemplateSpec, _Mapping]] = ...) -> None: ...

class BootstrapSpec(_message.Message):
    __slots__ = ["user_name", "user_password", "db_name"]
    USER_NAME_FIELD_NUMBER: _ClassVar[int]
    USER_PASSWORD_FIELD_NUMBER: _ClassVar[int]
    DB_NAME_FIELD_NUMBER: _ClassVar[int]
    user_name: str
    user_password: str
    db_name: str
    def __init__(self, user_name: _Optional[str] = ..., user_password: _Optional[str] = ..., db_name: _Optional[str] = ...) -> None: ...

class BackupSpec(_message.Message):
    __slots__ = ["backup_window_start", "retention_policy"]
    BACKUP_WINDOW_START_FIELD_NUMBER: _ClassVar[int]
    RETENTION_POLICY_FIELD_NUMBER: _ClassVar[int]
    backup_window_start: str
    retention_policy: str
    def __init__(self, backup_window_start: _Optional[str] = ..., retention_policy: _Optional[str] = ...) -> None: ...



"""Thin wrappers around the underlying gRPC channel used by the SDK.

This module exposes two small helper classes:

- :class:`AddressChannel` pairs a :class:`grpc.aio.Channel` with the
  resolved address string used to create it. The SDK uses this wrapper to
  keep track of which transport channel corresponds to which logical
  endpoint.
- :class:`ChannelBase` is a trivial subclass of the gRPC channel type used
  to annotate and accept SDK-style channels where a :class:`grpc.aio.Channel`
  is expected.
"""

from grpc.aio import Channel as GRPCChannel


class AddressChannel:
    """Simple container for a gRPC channel and its resolved address.

    :ivar address: Resolved address string used to create the channel.
    :type address: str
    :ivar channel: The underlying gRPC channel instance.
    :type channel: :class:`grpc.aio.Channel`

    :param channel: The underlying :class:`grpc.aio.Channel` instance.
    :param address: The resolved address string (for example ``'host:port'``)
      that was used to create the channel.
    """

    address: str
    channel: GRPCChannel

    def __init__(self, channel: GRPCChannel, address: str) -> None:
        """Initialize an :class:`AddressChannel` instance."""
        self.address = address
        self.channel = channel


class ChannelBase(GRPCChannel):
    """Base class used for SDK channel implementations.

    This trivial subclass exists primarily for type clarity: SDK components
    can accept a :class:`ChannelBase` to indicate they expect a gRPC channel
    implementing the SDK's extended behavior.
    """

    pass



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: nebius/vpc/v1alpha1/network_interface.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from nebius.api.buf.validate import validate_pb2 as buf_dot_validate_dot_validate__pb2
from nebius.api.nebius import annotations_pb2 as nebius_dot_annotations__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n+nebius/vpc/v1alpha1/network_interface.proto\x12\x13nebius.vpc.v1alpha1\x1a\x1b\x62uf/validate/validate.proto\x1a\x18nebius/annotations.proto\"\xcc\x02\n\x14NetworkInterfaceSpec\x12\'\n\tsubnet_id\x18\x01 \x01(\tB\n\xbaH\x03\xc8\x01\x01\xbaJ\x01\x02R\x08subnetId\x12\x36\n\x04name\x18\x02 \x01(\tB\"\xbaH\x1f\xc8\x01\x01r\x1a\x10\x01\x18<2\x14^[0-9a-z][0-9a-z-]*$R\x04name\x12\x43\n\nip_address\x18\x03 \x01(\x0b\x32\x1e.nebius.vpc.v1alpha1.IPAddressB\x04\xbaJ\x01\x06R\tipAddress\x12V\n\x11public_ip_address\x18\x04 \x01(\x0b\x32$.nebius.vpc.v1alpha1.PublicIPAddressB\x04\xbaJ\x01\x06R\x0fpublicIpAddress\x12\x36\n\x07\x61liases\x18\x06 \x03(\x0b\x32\x1c.nebius.vpc.v1alpha1.IPAliasR\x07\x61liases\"0\n\tIPAddress\x12#\n\rallocation_id\x18\x01 \x01(\tR\x0c\x61llocationId\"^\n\x0fPublicIPAddress\x12%\n\rallocation_id\x18\x01 \x01(\tH\x00R\x0c\x61llocationId\x12\x16\n\x06static\x18\x03 \x01(\x08R\x06staticB\x0c\n\nallocation\"6\n\x07IPAlias\x12+\n\rallocation_id\x18\x01 \x01(\tB\x06\xbaH\x03\xc8\x01\x01R\x0c\x61llocationId\"\xc0\x02\n\x16NetworkInterfaceStatus\x12\x14\n\x05index\x18\x01 \x01(\x05R\x05index\x12\x12\n\x04name\x18\x02 \x01(\tR\x04name\x12\x43\n\nip_address\x18\x03 \x01(\x0b\x32$.nebius.vpc.v1alpha1.IPAddressStatusR\tipAddress\x12V\n\x11public_ip_address\x18\x04 \x01(\x0b\x32*.nebius.vpc.v1alpha1.PublicIPAddressStatusR\x0fpublicIpAddress\x12>\n\x07\x61liases\x18\x06 \x01(\x0b\x32$.nebius.vpc.v1alpha1.IPAliasesStatusR\x07\x61liases\x12\x1f\n\x0bmac_address\x18\x07 \x01(\tR\nmacAddress\"P\n\x0fIPAddressStatus\x12\x18\n\x07\x61\x64\x64ress\x18\x01 \x01(\tR\x07\x61\x64\x64ress\x12#\n\rallocation_id\x18\x02 \x01(\tR\x0c\x61llocationId\"V\n\x15PublicIPAddressStatus\x12\x18\n\x07\x61\x64\x64ress\x18\x01 \x01(\tR\x07\x61\x64\x64ress\x12#\n\rallocation_id\x18\x02 \x01(\tR\x0c\x61llocationId\"\'\n\x0fIPAliasesStatus\x12\x14\n\x05\x63idrs\x18\x02 \x03(\tR\x05\x63idrsBh\n\x1a\x61i.nebius.pub.vpc.v1alpha1B\x15NetworkInterfaceProtoP\x01Z1github.com/nebius/gosdk/proto/nebius/vpc/v1alpha1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'nebius.vpc.v1alpha1.network_interface_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\032ai.nebius.pub.vpc.v1alpha1B\025NetworkInterfaceProtoP\001Z1github.com/nebius/gosdk/proto/nebius/vpc/v1alpha1'
  _NETWORKINTERFACESPEC.fields_by_name['subnet_id']._options = None
  _NETWORKINTERFACESPEC.fields_by_name['subnet_id']._serialized_options = b'\272H\003\310\001\001\272J\001\002'
  _NETWORKINTERFACESPEC.fields_by_name['name']._options = None
  _NETWORKINTERFACESPEC.fields_by_name['name']._serialized_options = b'\272H\037\310\001\001r\032\020\001\030<2\024^[0-9a-z][0-9a-z-]*$'
  _NETWORKINTERFACESPEC.fields_by_name['ip_address']._options = None
  _NETWORKINTERFACESPEC.fields_by_name['ip_address']._serialized_options = b'\272J\001\006'
  _NETWORKINTERFACESPEC.fields_by_name['public_ip_address']._options = None
  _NETWORKINTERFACESPEC.fields_by_name['public_ip_address']._serialized_options = b'\272J\001\006'
  _IPALIAS.fields_by_name['allocation_id']._options = None
  _IPALIAS.fields_by_name['allocation_id']._serialized_options = b'\272H\003\310\001\001'
  _globals['_NETWORKINTERFACESPEC']._serialized_start=124
  _globals['_NETWORKINTERFACESPEC']._serialized_end=456
  _globals['_IPADDRESS']._serialized_start=458
  _globals['_IPADDRESS']._serialized_end=506
  _globals['_PUBLICIPADDRESS']._serialized_start=508
  _globals['_PUBLICIPADDRESS']._serialized_end=602
  _globals['_IPALIAS']._serialized_start=604
  _globals['_IPALIAS']._serialized_end=658
  _globals['_NETWORKINTERFACESTATUS']._serialized_start=661
  _globals['_NETWORKINTERFACESTATUS']._serialized_end=981
  _globals['_IPADDRESSSTATUS']._serialized_start=983
  _globals['_IPADDRESSSTATUS']._serialized_end=1063
  _globals['_PUBLICIPADDRESSSTATUS']._serialized_start=1065
  _globals['_PUBLICIPADDRESSSTATUS']._serialized_end=1151
  _globals['_IPALIASESSTATUS']._serialized_start=1153
  _globals['_IPALIASESSTATUS']._serialized_end=1192
# @@protoc_insertion_point(module_scope)



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from nebius.api.nebius.common.v1 import operation_pb2 as nebius_dot_common_dot_v1_dot_operation__pb2
from nebius.api.nebius.iam.v2 import access_key_pb2 as nebius_dot_iam_dot_v2_dot_access__key__pb2
from nebius.api.nebius.iam.v2 import access_key_service_pb2 as nebius_dot_iam_dot_v2_dot_access__key__service__pb2


class AccessKeyServiceStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.Create = channel.unary_unary(
                '/nebius.iam.v2.AccessKeyService/Create',
                request_serializer=nebius_dot_iam_dot_v2_dot_access__key__service__pb2.CreateAccessKeyRequest.SerializeToString,
                response_deserializer=nebius_dot_common_dot_v1_dot_operation__pb2.Operation.FromString,
                )
        self.Get = channel.unary_unary(
                '/nebius.iam.v2.AccessKeyService/Get',
                request_serializer=nebius_dot_iam_dot_v2_dot_access__key__service__pb2.GetAccessKeyRequest.SerializeToString,
                response_deserializer=nebius_dot_iam_dot_v2_dot_access__key__pb2.AccessKey.FromString,
                )
        self.GetSecret = channel.unary_unary(
                '/nebius.iam.v2.AccessKeyService/GetSecret',
                request_serializer=nebius_dot_iam_dot_v2_dot_access__key__service__pb2.GetAccessKeySecretRequest.SerializeToString,
                response_deserializer=nebius_dot_iam_dot_v2_dot_access__key__service__pb2.GetAccessKeySecretResponse.FromString,
                )
        self.List = channel.unary_unary(
                '/nebius.iam.v2.AccessKeyService/List',
                request_serializer=nebius_dot_iam_dot_v2_dot_access__key__service__pb2.ListAccessKeysRequest.SerializeToString,
                response_deserializer=nebius_dot_iam_dot_v2_dot_access__key__service__pb2.ListAccessKeysResponse.FromString,
                )
        self.Update = channel.unary_unary(
                '/nebius.iam.v2.AccessKeyService/Update',
                request_serializer=nebius_dot_iam_dot_v2_dot_access__key__service__pb2.UpdateAccessKeyRequest.SerializeToString,
                response_deserializer=nebius_dot_common_dot_v1_dot_operation__pb2.Operation.FromString,
                )
        self.Delete = channel.unary_unary(
                '/nebius.iam.v2.AccessKeyService/Delete',
                request_serializer=nebius_dot_iam_dot_v2_dot_access__key__service__pb2.DeleteAccessKeyRequest.SerializeToString,
                response_deserializer=nebius_dot_common_dot_v1_dot_operation__pb2.Operation.FromString,
                )
        self.Activate = channel.unary_unary(
                '/nebius.iam.v2.AccessKeyService/Activate',
                request_serializer=nebius_dot_iam_dot_v2_dot_access__key__service__pb2.ActivateAccessKeyRequest.SerializeToString,
                response_deserializer=nebius_dot_common_dot_v1_dot_operation__pb2.Operation.FromString,
                )
        self.Deactivate = channel.unary_unary(
                '/nebius.iam.v2.AccessKeyService/Deactivate',
                request_serializer=nebius_dot_iam_dot_v2_dot_access__key__service__pb2.DeactivateAccessKeyRequest.SerializeToString,
                response_deserializer=nebius_dot_common_dot_v1_dot_operation__pb2.Operation.FromString,
                )
        self.ListByAccount = channel.unary_unary(
                '/nebius.iam.v2.AccessKeyService/ListByAccount',
                request_serializer=nebius_dot_iam_dot_v2_dot_access__key__service__pb2.ListAccessKeysByAccountRequest.SerializeToString,
                response_deserializer=nebius_dot_iam_dot_v2_dot_access__key__service__pb2.ListAccessKeysResponse.FromString,
                )
        self.GetByAwsId = channel.unary_unary(
                '/nebius.iam.v2.AccessKeyService/GetByAwsId',
                request_serializer=nebius_dot_iam_dot_v2_dot_access__key__service__pb2.GetAccessKeyByAwsIdRequest.SerializeToString,
                response_deserializer=nebius_dot_iam_dot_v2_dot_access__key__pb2.AccessKey.FromString,
                )
        self.DeleteByAwsId = channel.unary_unary(
                '/nebius.iam.v2.AccessKeyService/DeleteByAwsId',
                request_serializer=nebius_dot_iam_dot_v2_dot_access__key__service__pb2.DeleteAccessKeyByAwsIdRequest.SerializeToString,
                response_deserializer=nebius_dot_common_dot_v1_dot_operation__pb2.Operation.FromString,
                )
        self.ActivateByAwsId = channel.unary_unary(
                '/nebius.iam.v2.AccessKeyService/ActivateByAwsId',
                request_serializer=nebius_dot_iam_dot_v2_dot_access__key__service__pb2.ActivateAccessKeyByAwsIdRequest.SerializeToString,
                response_deserializer=nebius_dot_common_dot_v1_dot_operation__pb2.Operation.FromString,
                )
        self.DeactivateByAwsId = channel.unary_unary(
                '/nebius.iam.v2.AccessKeyService/DeactivateByAwsId',
                request_serializer=nebius_dot_iam_dot_v2_dot_access__key__service__pb2.DeactivateAccessKeyByAwsIdRequest.SerializeToString,
                response_deserializer=nebius_dot_common_dot_v1_dot_operation__pb2.Operation.FromString,
                )


class AccessKeyServiceServicer(object):
    """Missing associated documentation comment in .proto file."""

    def Create(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Get(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetSecret(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def List(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Update(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Delete(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Activate(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Deactivate(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ListByAccount(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetByAwsId(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def DeleteByAwsId(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ActivateByAwsId(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def DeactivateByAwsId(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_AccessKeyServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'Create': grpc.unary_unary_rpc_method_handler(
                    servicer.Create,
                    request_deserializer=nebius_dot_iam_dot_v2_dot_access__key__service__pb2.CreateAccessKeyRequest.FromString,
                    response_serializer=nebius_dot_common_dot_v1_dot_operation__pb2.Operation.SerializeToString,
            ),
            'Get': grpc.unary_unary_rpc_method_handler(
                    servicer.Get,
                    request_deserializer=nebius_dot_iam_dot_v2_dot_access__key__service__pb2.GetAccessKeyRequest.FromString,
                    response_serializer=nebius_dot_iam_dot_v2_dot_access__key__pb2.AccessKey.SerializeToString,
            ),
            'GetSecret': grpc.unary_unary_rpc_method_handler(
                    servicer.GetSecret,
                    request_deserializer=nebius_dot_iam_dot_v2_dot_access__key__service__pb2.GetAccessKeySecretRequest.FromString,
                    response_serializer=nebius_dot_iam_dot_v2_dot_access__key__service__pb2.GetAccessKeySecretResponse.SerializeToString,
            ),
            'List': grpc.unary_unary_rpc_method_handler(
                    servicer.List,
                    request_deserializer=nebius_dot_iam_dot_v2_dot_access__key__service__pb2.ListAccessKeysRequest.FromString,
                    response_serializer=nebius_dot_iam_dot_v2_dot_access__key__service__pb2.ListAccessKeysResponse.SerializeToString,
            ),
            'Update': grpc.unary_unary_rpc_method_handler(
                    servicer.Update,
                    request_deserializer=nebius_dot_iam_dot_v2_dot_access__key__service__pb2.UpdateAccessKeyRequest.FromString,
                    response_serializer=nebius_dot_common_dot_v1_dot_operation__pb2.Operation.SerializeToString,
            ),
            'Delete': grpc.unary_unary_rpc_method_handler(
                    servicer.Delete,
                    request_deserializer=nebius_dot_iam_dot_v2_dot_access__key__service__pb2.DeleteAccessKeyRequest.FromString,
                    response_serializer=nebius_dot_common_dot_v1_dot_operation__pb2.Operation.SerializeToString,
            ),
            'Activate': grpc.unary_unary_rpc_method_handler(
                    servicer.Activate,
                    request_deserializer=nebius_dot_iam_dot_v2_dot_access__key__service__pb2.ActivateAccessKeyRequest.FromString,
                    response_serializer=nebius_dot_common_dot_v1_dot_operation__pb2.Operation.SerializeToString,
            ),
            'Deactivate': grpc.unary_unary_rpc_method_handler(
                    servicer.Deactivate,
                    request_deserializer=nebius_dot_iam_dot_v2_dot_access__key__service__pb2.DeactivateAccessKeyRequest.FromString,
                    response_serializer=nebius_dot_common_dot_v1_dot_operation__pb2.Operation.SerializeToString,
            ),
            'ListByAccount': grpc.unary_unary_rpc_method_handler(
                    servicer.ListByAccount,
                    request_deserializer=nebius_dot_iam_dot_v2_dot_access__key__service__pb2.ListAccessKeysByAccountRequest.FromString,
                    response_serializer=nebius_dot_iam_dot_v2_dot_access__key__service__pb2.ListAccessKeysResponse.SerializeToString,
            ),
            'GetByAwsId': grpc.unary_unary_rpc_method_handler(
                    servicer.GetByAwsId,
                    request_deserializer=nebius_dot_iam_dot_v2_dot_access__key__service__pb2.GetAccessKeyByAwsIdRequest.FromString,
                    response_serializer=nebius_dot_iam_dot_v2_dot_access__key__pb2.AccessKey.SerializeToString,
            ),
            'DeleteByAwsId': grpc.unary_unary_rpc_method_handler(
                    servicer.DeleteByAwsId,
                    request_deserializer=nebius_dot_iam_dot_v2_dot_access__key__service__pb2.DeleteAccessKeyByAwsIdRequest.FromString,
                    response_serializer=nebius_dot_common_dot_v1_dot_operation__pb2.Operation.SerializeToString,
            ),
            'ActivateByAwsId': grpc.unary_unary_rpc_method_handler(
                    servicer.ActivateByAwsId,
                    request_deserializer=nebius_dot_iam_dot_v2_dot_access__key__service__pb2.ActivateAccessKeyByAwsIdRequest.FromString,
                    response_serializer=nebius_dot_common_dot_v1_dot_operation__pb2.Operation.SerializeToString,
            ),
            'DeactivateByAwsId': grpc.unary_unary_rpc_method_handler(
                    servicer.DeactivateByAwsId,
                    request_deserializer=nebius_dot_iam_dot_v2_dot_access__key__service__pb2.DeactivateAccessKeyByAwsIdRequest.FromString,
                    response_serializer=nebius_dot_common_dot_v1_dot_operation__pb2.Operation.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'nebius.iam.v2.AccessKeyService', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class AccessKeyService(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def Create(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nebius.iam.v2.AccessKeyService/Create',
            nebius_dot_iam_dot_v2_dot_access__key__service__pb2.CreateAccessKeyRequest.SerializeToString,
            nebius_dot_common_dot_v1_dot_operation__pb2.Operation.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Get(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nebius.iam.v2.AccessKeyService/Get',
            nebius_dot_iam_dot_v2_dot_access__key__service__pb2.GetAccessKeyRequest.SerializeToString,
            nebius_dot_iam_dot_v2_dot_access__key__pb2.AccessKey.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetSecret(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nebius.iam.v2.AccessKeyService/GetSecret',
            nebius_dot_iam_dot_v2_dot_access__key__service__pb2.GetAccessKeySecretRequest.SerializeToString,
            nebius_dot_iam_dot_v2_dot_access__key__service__pb2.GetAccessKeySecretResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def List(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nebius.iam.v2.AccessKeyService/List',
            nebius_dot_iam_dot_v2_dot_access__key__service__pb2.ListAccessKeysRequest.SerializeToString,
            nebius_dot_iam_dot_v2_dot_access__key__service__pb2.ListAccessKeysResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Update(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nebius.iam.v2.AccessKeyService/Update',
            nebius_dot_iam_dot_v2_dot_access__key__service__pb2.UpdateAccessKeyRequest.SerializeToString,
            nebius_dot_common_dot_v1_dot_operation__pb2.Operation.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Delete(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nebius.iam.v2.AccessKeyService/Delete',
            nebius_dot_iam_dot_v2_dot_access__key__service__pb2.DeleteAccessKeyRequest.SerializeToString,
            nebius_dot_common_dot_v1_dot_operation__pb2.Operation.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Activate(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nebius.iam.v2.AccessKeyService/Activate',
            nebius_dot_iam_dot_v2_dot_access__key__service__pb2.ActivateAccessKeyRequest.SerializeToString,
            nebius_dot_common_dot_v1_dot_operation__pb2.Operation.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Deactivate(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nebius.iam.v2.AccessKeyService/Deactivate',
            nebius_dot_iam_dot_v2_dot_access__key__service__pb2.DeactivateAccessKeyRequest.SerializeToString,
            nebius_dot_common_dot_v1_dot_operation__pb2.Operation.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ListByAccount(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nebius.iam.v2.AccessKeyService/ListByAccount',
            nebius_dot_iam_dot_v2_dot_access__key__service__pb2.ListAccessKeysByAccountRequest.SerializeToString,
            nebius_dot_iam_dot_v2_dot_access__key__service__pb2.ListAccessKeysResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetByAwsId(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nebius.iam.v2.AccessKeyService/GetByAwsId',
            nebius_dot_iam_dot_v2_dot_access__key__service__pb2.GetAccessKeyByAwsIdRequest.SerializeToString,
            nebius_dot_iam_dot_v2_dot_access__key__pb2.AccessKey.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def DeleteByAwsId(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nebius.iam.v2.AccessKeyService/DeleteByAwsId',
            nebius_dot_iam_dot_v2_dot_access__key__service__pb2.DeleteAccessKeyByAwsIdRequest.SerializeToString,
            nebius_dot_common_dot_v1_dot_operation__pb2.Operation.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ActivateByAwsId(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nebius.iam.v2.AccessKeyService/ActivateByAwsId',
            nebius_dot_iam_dot_v2_dot_access__key__service__pb2.ActivateAccessKeyByAwsIdRequest.SerializeToString,
            nebius_dot_common_dot_v1_dot_operation__pb2.Operation.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def DeactivateByAwsId(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nebius.iam.v2.AccessKeyService/DeactivateByAwsId',
            nebius_dot_iam_dot_v2_dot_access__key__service__pb2.DeactivateAccessKeyByAwsIdRequest.SerializeToString,
            nebius_dot_common_dot_v1_dot_operation__pb2.Operation.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: nebius/mysterybox/v1/secret_service.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from nebius.api.nebius.mysterybox.v1 import secret_pb2 as nebius_dot_mysterybox_dot_v1_dot_secret__pb2
from nebius.api.nebius.common.v1 import metadata_pb2 as nebius_dot_common_dot_v1_dot_metadata__pb2
from nebius.api.nebius.common.v1 import operation_pb2 as nebius_dot_common_dot_v1_dot_operation__pb2
from nebius.api.buf.validate import validate_pb2 as buf_dot_validate_dot_validate__pb2
from nebius.api.nebius import annotations_pb2 as nebius_dot_annotations__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n)nebius/mysterybox/v1/secret_service.proto\x12\x14nebius.mysterybox.v1\x1a!nebius/mysterybox/v1/secret.proto\x1a\x1fnebius/common/v1/metadata.proto\x1a nebius/common/v1/operation.proto\x1a\x1b\x62uf/validate/validate.proto\x1a\x18nebius/annotations.proto\"\x8b\x01\n\x13\x43reateSecretRequest\x12>\n\x08metadata\x18\x01 \x01(\x0b\x32\".nebius.common.v1.ResourceMetadataR\x08metadata\x12\x34\n\x04spec\x18\x02 \x01(\x0b\x32 .nebius.mysterybox.v1.SecretSpecR\x04spec\"\x8b\x01\n\x13UpdateSecretRequest\x12>\n\x08metadata\x18\x01 \x01(\x0b\x32\".nebius.common.v1.ResourceMetadataR\x08metadata\x12\x34\n\x04spec\x18\x02 \x01(\x0b\x32 .nebius.mysterybox.v1.SecretSpecR\x04spec\"i\n\x10GetSecretRequest\x12\x16\n\x02id\x18\x01 \x01(\tB\x06\xbaH\x03\xc8\x01\x01R\x02id\x12=\n\x1bshow_scheduled_for_deletion\x18\x02 \x01(\x08R\x18showScheduledForDeletion\"_\n\x16GetSecretByNameRequest\x12#\n\tparent_id\x18\x01 \x01(\tB\x06\xbaH\x03\xc8\x01\x01R\x08parentId\x12\x1a\n\x04name\x18\x02 \x01(\tB\x06\xbaH\x03\xc8\x01\x01R\x04nameJ\x04\x08\x03\x10\x04\"\xc3\x01\n\x12ListSecretsRequest\x12#\n\tparent_id\x18\x01 \x01(\tB\x06\xbaH\x03\xc8\x01\x01R\x08parentId\x12*\n\tpage_size\x18\x02 \x01(\x03\x42\r\xbaH\n\xd8\x01\x01\"\x05\x18\xe8\x07(\x00R\x08pageSize\x12\x1d\n\npage_token\x18\x03 \x01(\tR\tpageToken\x12=\n\x1bshow_scheduled_for_deletion\x18\x04 \x01(\x08R\x18showScheduledForDeletion\"q\n\x13ListSecretsResponse\x12&\n\x0fnext_page_token\x18\x02 \x01(\tR\rnextPageToken\x12\x32\n\x05items\x18\x03 \x03(\x0b\x32\x1c.nebius.mysterybox.v1.SecretR\x05items\"-\n\x13\x44\x65leteSecretRequest\x12\x16\n\x02id\x18\x01 \x01(\tB\x06\xbaH\x03\xc8\x01\x01R\x02id\"C\n\x15UndeleteSecretRequest\x12\x16\n\x02id\x18\x01 \x01(\tB\x06\xbaH\x03\xc8\x01\x01R\x02id\x12\x12\n\x04name\x18\x02 \x01(\tR\x04name2\xf1\x04\n\rSecretService\x12P\n\x06\x43reate\x12).nebius.mysterybox.v1.CreateSecretRequest\x1a\x1b.nebius.common.v1.Operation\x12P\n\x06Update\x12).nebius.mysterybox.v1.UpdateSecretRequest\x1a\x1b.nebius.common.v1.Operation\x12K\n\x03Get\x12&.nebius.mysterybox.v1.GetSecretRequest\x1a\x1c.nebius.mysterybox.v1.Secret\x12W\n\tGetByName\x12,.nebius.mysterybox.v1.GetSecretByNameRequest\x1a\x1c.nebius.mysterybox.v1.Secret\x12[\n\x04List\x12(.nebius.mysterybox.v1.ListSecretsRequest\x1a).nebius.mysterybox.v1.ListSecretsResponse\x12P\n\x06\x44\x65lete\x12).nebius.mysterybox.v1.DeleteSecretRequest\x1a\x1b.nebius.common.v1.Operation\x12T\n\x08Undelete\x12+.nebius.mysterybox.v1.UndeleteSecretRequest\x1a\x1b.nebius.common.v1.Operation\x1a\x11\xbaJ\x0e\x63pl.mysteryboxBg\n\x1b\x61i.nebius.pub.mysterybox.v1B\x12SecretServiceProtoP\x01Z2github.com/nebius/gosdk/proto/nebius/mysterybox/v1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'nebius.mysterybox.v1.secret_service_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\033ai.nebius.pub.mysterybox.v1B\022SecretServiceProtoP\001Z2github.com/nebius/gosdk/proto/nebius/mysterybox/v1'
  _GETSECRETREQUEST.fields_by_name['id']._options = None
  _GETSECRETREQUEST.fields_by_name['id']._serialized_options = b'\272H\003\310\001\001'
  _GETSECRETBYNAMEREQUEST.fields_by_name['parent_id']._options = None
  _GETSECRETBYNAMEREQUEST.fields_by_name['parent_id']._serialized_options = b'\272H\003\310\001\001'
  _GETSECRETBYNAMEREQUEST.fields_by_name['name']._options = None
  _GETSECRETBYNAMEREQUEST.fields_by_name['name']._serialized_options = b'\272H\003\310\001\001'
  _LISTSECRETSREQUEST.fields_by_name['parent_id']._options = None
  _LISTSECRETSREQUEST.fields_by_name['parent_id']._serialized_options = b'\272H\003\310\001\001'
  _LISTSECRETSREQUEST.fields_by_name['page_size']._options = None
  _LISTSECRETSREQUEST.fields_by_name['page_size']._serialized_options = b'\272H\n\330\001\001\"\005\030\350\007(\000'
  _DELETESECRETREQUEST.fields_by_name['id']._options = None
  _DELETESECRETREQUEST.fields_by_name['id']._serialized_options = b'\272H\003\310\001\001'
  _UNDELETESECRETREQUEST.fields_by_name['id']._options = None
  _UNDELETESECRETREQUEST.fields_by_name['id']._serialized_options = b'\272H\003\310\001\001'
  _SECRETSERVICE._options = None
  _SECRETSERVICE._serialized_options = b'\272J\016cpl.mysterybox'
  _globals['_CREATESECRETREQUEST']._serialized_start=225
  _globals['_CREATESECRETREQUEST']._serialized_end=364
  _globals['_UPDATESECRETREQUEST']._serialized_start=367
  _globals['_UPDATESECRETREQUEST']._serialized_end=506
  _globals['_GETSECRETREQUEST']._serialized_start=508
  _globals['_GETSECRETREQUEST']._serialized_end=613
  _globals['_GETSECRETBYNAMEREQUEST']._serialized_start=615
  _globals['_GETSECRETBYNAMEREQUEST']._serialized_end=710
  _globals['_LISTSECRETSREQUEST']._serialized_start=713
  _globals['_LISTSECRETSREQUEST']._serialized_end=908
  _globals['_LISTSECRETSRESPONSE']._serialized_start=910
  _globals['_LISTSECRETSRESPONSE']._serialized_end=1023
  _globals['_DELETESECRETREQUEST']._serialized_start=1025
  _globals['_DELETESECRETREQUEST']._serialized_end=1070
  _globals['_UNDELETESECRETREQUEST']._serialized_start=1072
  _globals['_UNDELETESECRETREQUEST']._serialized_end=1139
  _globals['_SECRETSERVICE']._serialized_start=1142
  _globals['_SECRETSERVICE']._serialized_end=1767
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: nebius/iam/v2/project_service.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from nebius.api.buf.validate import validate_pb2 as buf_dot_validate_dot_validate__pb2
from nebius.api.nebius import annotations_pb2 as nebius_dot_annotations__pb2
from nebius.api.nebius.common.v1 import metadata_pb2 as nebius_dot_common_dot_v1_dot_metadata__pb2
from nebius.api.nebius.common.v1 import operation_pb2 as nebius_dot_common_dot_v1_dot_operation__pb2
from nebius.api.nebius.iam.v2 import project_pb2 as nebius_dot_iam_dot_v2_dot_project__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n#nebius/iam/v2/project_service.proto\x12\rnebius.iam.v2\x1a\x1b\x62uf/validate/validate.proto\x1a\x18nebius/annotations.proto\x1a\x1fnebius/common/v1/metadata.proto\x1a nebius/common/v1/operation.proto\x1a\x1bnebius/iam/v2/project.proto\"\x86\x01\n\x14\x43reateProjectRequest\x12>\n\x08metadata\x18\x01 \x01(\x0b\x32\".nebius.common.v1.ResourceMetadataR\x08metadata\x12.\n\x04spec\x18\x02 \x01(\x0b\x32\x1a.nebius.iam.v2.ProjectSpecR\x04spec\"#\n\x11GetProjectRequest\x12\x0e\n\x02id\x18\x01 \x01(\tR\x02id\"Z\n\x17GetProjectByNameRequest\x12#\n\tparent_id\x18\x01 \x01(\tB\x06\xbaH\x03\xc8\x01\x01R\x08parentId\x12\x1a\n\x04name\x18\x02 \x01(\tB\x06\xbaH\x03\xc8\x01\x01R\x04name\"\x86\x01\n\x13ListProjectsRequest\x12\x1b\n\tparent_id\x18\x01 \x01(\tR\x08parentId\x12\x1b\n\tpage_size\x18\x02 \x01(\x03R\x08pageSize\x12\x1d\n\npage_token\x18\x03 \x01(\tR\tpageToken\x12\x16\n\x06\x66ilter\x18\x04 \x01(\tR\x06\x66ilter\"\x86\x01\n\x14UpdateProjectRequest\x12>\n\x08metadata\x18\x01 \x01(\x0b\x32\".nebius.common.v1.ResourceMetadataR\x08metadata\x12.\n\x04spec\x18\x02 \x01(\x0b\x32\x1a.nebius.iam.v2.ProjectSpecR\x04spec\"&\n\x14\x44\x65leteProjectRequest\x12\x0e\n\x02id\x18\x01 \x01(\tR\x02id\"l\n\x14ListProjectsResponse\x12,\n\x05items\x18\x01 \x03(\x0b\x32\x16.nebius.iam.v2.ProjectR\x05items\x12&\n\x0fnext_page_token\x18\x02 \x01(\tR\rnextPageToken2\xdf\x03\n\x0eProjectService\x12J\n\x06\x43reate\x12#.nebius.iam.v2.CreateProjectRequest\x1a\x1b.nebius.common.v1.Operation\x12?\n\x03Get\x12 .nebius.iam.v2.GetProjectRequest\x1a\x16.nebius.iam.v2.Project\x12K\n\tGetByName\x12&.nebius.iam.v2.GetProjectByNameRequest\x1a\x16.nebius.iam.v2.Project\x12O\n\x04List\x12\".nebius.iam.v2.ListProjectsRequest\x1a#.nebius.iam.v2.ListProjectsResponse\x12J\n\x06Update\x12#.nebius.iam.v2.UpdateProjectRequest\x1a\x1b.nebius.common.v1.Operation\x12J\n\x06\x44\x65lete\x12#.nebius.iam.v2.DeleteProjectRequest\x1a\x1b.nebius.common.v1.Operation\x1a\n\xbaJ\x07\x63pl.iamBZ\n\x14\x61i.nebius.pub.iam.v2B\x13ProjectServiceProtoP\x01Z+github.com/nebius/gosdk/proto/nebius/iam/v2b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'nebius.iam.v2.project_service_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\024ai.nebius.pub.iam.v2B\023ProjectServiceProtoP\001Z+github.com/nebius/gosdk/proto/nebius/iam/v2'
  _GETPROJECTBYNAMEREQUEST.fields_by_name['parent_id']._options = None
  _GETPROJECTBYNAMEREQUEST.fields_by_name['parent_id']._serialized_options = b'\272H\003\310\001\001'
  _GETPROJECTBYNAMEREQUEST.fields_by_name['name']._options = None
  _GETPROJECTBYNAMEREQUEST.fields_by_name['name']._serialized_options = b'\272H\003\310\001\001'
  _PROJECTSERVICE._options = None
  _PROJECTSERVICE._serialized_options = b'\272J\007cpl.iam'
  _globals['_CREATEPROJECTREQUEST']._serialized_start=206
  _globals['_CREATEPROJECTREQUEST']._serialized_end=340
  _globals['_GETPROJECTREQUEST']._serialized_start=342
  _globals['_GETPROJECTREQUEST']._serialized_end=377
  _globals['_GETPROJECTBYNAMEREQUEST']._serialized_start=379
  _globals['_GETPROJECTBYNAMEREQUEST']._serialized_end=469
  _globals['_LISTPROJECTSREQUEST']._serialized_start=472
  _globals['_LISTPROJECTSREQUEST']._serialized_end=606
  _globals['_UPDATEPROJECTREQUEST']._serialized_start=609
  _globals['_UPDATEPROJECTREQUEST']._serialized_end=743
  _globals['_DELETEPROJECTREQUEST']._serialized_start=745
  _globals['_DELETEPROJECTREQUEST']._serialized_end=783
  _globals['_LISTPROJECTSRESPONSE']._serialized_start=785
  _globals['_LISTPROJECTSRESPONSE']._serialized_end=893
  _globals['_PROJECTSERVICE']._serialized_start=896
  _globals['_PROJECTSERVICE']._serialized_end=1375
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: nebius/iam/v1/token_exchange_service.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from nebius.api.nebius import annotations_pb2 as nebius_dot_annotations__pb2
from nebius.api.nebius.iam.v1 import token_service_pb2 as nebius_dot_iam_dot_v1_dot_token__service__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n*nebius/iam/v1/token_exchange_service.proto\x12\rnebius.iam.v1\x1a\x18nebius/annotations.proto\x1a!nebius/iam/v1/token_service.proto2z\n\x14TokenExchangeService\x12S\n\x08\x45xchange\x12#.nebius.iam.v1.ExchangeTokenRequest\x1a\".nebius.iam.v1.CreateTokenResponse\x1a\r\xbaJ\ntokens.iamB`\n\x14\x61i.nebius.pub.iam.v1B\x19TokenExchangeServiceProtoP\x01Z+github.com/nebius/gosdk/proto/nebius/iam/v1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'nebius.iam.v1.token_exchange_service_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\024ai.nebius.pub.iam.v1B\031TokenExchangeServiceProtoP\001Z+github.com/nebius/gosdk/proto/nebius/iam/v1'
  _TOKENEXCHANGESERVICE._options = None
  _TOKENEXCHANGESERVICE._serialized_options = b'\272J\ntokens.iam'
  _globals['_TOKENEXCHANGESERVICE']._serialized_start=122
  _globals['_TOKENEXCHANGESERVICE']._serialized_end=244
# @@protoc_insertion_point(module_scope)



"""Throttled token cache helper.

This module provides :class:`ThrottledTokenCache`, a thin wrapper around
:class:`TokenCache` which prevents frequent disk reads by caching the
value in-memory for a configurable throttle period.

The class is useful when many coroutines may request the same token
frequently and you want to reduce filesystem and YAML parsing overhead.

Example
-------

    cache = ThrottledTokenCache("service-account")
    tok = await cache.get()

"""

from datetime import datetime, timedelta, timezone
from logging import getLogger
from pathlib import Path

from nebius.aio.token.token import Token
from nebius.base.constants import DEFAULT_CONFIG_DIR, DEFAULT_CREDENTIALS_FILE

from .token_cache import TokenCache

log = getLogger(__name__)


class ThrottledTokenCache:
    """A throttled file-based token cache.

    The helper keeps an in-memory cached token for ``throttle`` seconds
    to avoid repeated reads from the underlying :class:`TokenCache`.

    :param name: The key name under which the token is stored in the
        underlying :class:`TokenCache`.
    :param cache_file: Path to the underlying YAML cache file.
    :param throttle: Duration (timedelta or seconds as float) to
        hold the in-memory cached token. If a float is supplied it is
        interpreted as seconds.
    """

    def __init__(
        self,
        name: str,
        cache_file: str | Path = Path(DEFAULT_CONFIG_DIR) / DEFAULT_CREDENTIALS_FILE,
        throttle: timedelta | float = timedelta(minutes=5),
    ) -> None:
        """Create a throttled token cache."""
        self._name = name
        self._cache = TokenCache(cache_file)
        if isinstance(throttle, (float, int)):
            throttle = timedelta(seconds=throttle)
        self._throttle: timedelta = throttle
        self._cached_token: Token | None = None
        self._next_check: datetime = datetime.now(timezone.utc)

    def get_cached(self) -> Token | None:
        """Return the in-memory cached token without consulting disk.

        :returns: The cached :class:`Token` or `None` if no token is cached.
        """
        return self._cached_token

    async def get(self) -> Token | None:
        """Return the cached token, refreshing from disk if throttling allows.

        If the in-memory token is present, not expired and the throttle
        period has not elapsed the cached token is returned. Otherwise
        the underlying :meth:`TokenCache.get` is called.

        :returns: The cached :class:`Token` or `None` if no valid token is found.
        """
        if (
            self._cached_token is not None
            and not self._cached_token.is_expired()
            and self._next_check > datetime.now(timezone.utc)
        ):
            return self._cached_token

        return await self.refresh()

    async def set(self, token: Token) -> None:
        """Store the token in the underlying cache and update the in-memory
        cache and throttle expiration.

        If the provided token equals the in-memory cached token no write is performed.
        """
        if self._cached_token == token:
            return
        await self._cache.set(self._name, token)
        self._cached_token = token
        self._next_check = datetime.now(timezone.utc) + self._throttle

    async def remove(self) -> None:
        """Remove the token from both the underlying cache and the in-memory cache."""
        await self._cache.remove(self._name)
        self._cached_token = None
        self._next_check = datetime.now(timezone.utc)

    async def remove_if_equal(self, token: Token) -> None:
        """Remove the token if it equals the provided token.

        This performs an atomic check-and-remove against the underlying
        cache and updates the in-memory state when the removal occurs.
        """
        await self._cache.remove_if_equal(self._name, token)
        if self._cached_token == token:
            self._cached_token = None
            self._next_check = datetime.now(timezone.utc)

    async def refresh(self) -> Token | None:
        """Refresh the in-memory token from the underlying cache.

        If the token exists on disk and is not expired the in-memory
        cache and throttle are updated and the token is returned.
        """
        token = await self._cache.get(self._name)
        if token is not None and not token.is_expired():
            self._cached_token = token
            self._next_check = datetime.now(timezone.utc) + self._throttle
            return token
        return None



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: nebius/logging/v1/agentmanager/version_service.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import duration_pb2 as google_dot_protobuf_dot_duration__pb2
from nebius.api.nebius import annotations_pb2 as nebius_dot_annotations__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n4nebius/logging/v1/agentmanager/version_service.proto\x12\x1enebius.logging.agentmanager.v1\x1a\x1egoogle/protobuf/duration.proto\x1a\x18nebius/annotations.proto\"\xf6\x07\n\x11GetVersionRequest\x12=\n\x04type\x18\x01 \x01(\x0e\x32).nebius.logging.agentmanager.v1.AgentTypeR\x04type\x12#\n\ragent_version\x18\x02 \x01(\tR\x0c\x61gentVersion\x12\'\n\x0fupdater_version\x18\x03 \x01(\tR\x0eupdaterVersion\x12\x1b\n\tparent_id\x18\x04 \x01(\tR\x08parentId\x12\x1f\n\x0binstance_id\x18\x05 \x01(\tR\ninstanceId\x12?\n\x07os_info\x18\x06 \x01(\x0b\x32&.nebius.logging.agentmanager.v1.OSInfoR\x06osInfo\x12K\n\x0b\x61gent_state\x18\x07 \x01(\x0e\x32*.nebius.logging.agentmanager.v1.AgentStateR\nagentState\x12<\n\x0c\x61gent_uptime\x18\x08 \x01(\x0b\x32\x19.google.protobuf.DurationR\x0b\x61gentUptime\x12>\n\rsystem_uptime\x18\t \x01(\x0b\x32\x19.google.protobuf.DurationR\x0csystemUptime\x12@\n\x0eupdater_uptime\x18\n \x01(\x0b\x32\x19.google.protobuf.DurationR\rupdaterUptime\x12\x30\n\x14\x61gent_state_messages\x18\x0b \x03(\tR\x12\x61gentStateMessages\x12*\n\x11last_update_error\x18\x0c \x01(\tR\x0flastUpdateError\x12&\n\x0fmk8s_cluster_id\x18\r \x01(\tR\rmk8sClusterId\x12T\n\x0emodules_health\x18\x0e \x01(\x0b\x32-.nebius.logging.agentmanager.v1.ModulesHealthR\rmodulesHealth\x12*\n\x11\x63loud_init_status\x18\x0f \x01(\tR\x0f\x63loudInitStatus\x12\x39\n\x19instance_id_used_fallback\x18\x10 \x01(\x08R\x16instanceIdUsedFallback\x12&\n\x0flast_agent_logs\x18\x11 \x01(\tR\rlastAgentLogs\x12\x1b\n\tgpu_model\x18\x12 \x01(\tR\x08gpuModel\x12\x1d\n\ngpu_number\x18\x13 \x01(\x05R\tgpuNumber\x12!\n\x0c\x64\x63gm_version\x18\x14 \x01(\tR\x0b\x64\x63gmVersion\"\xa7\x03\n\rModulesHealth\x12\x46\n\x07process\x18\x01 \x01(\x0b\x32,.nebius.logging.agentmanager.v1.ModuleHealthR\x07process\x12O\n\x0cgpu_pipeline\x18\x02 \x01(\x0b\x32,.nebius.logging.agentmanager.v1.ModuleHealthR\x0bgpuPipeline\x12O\n\x0c\x63pu_pipeline\x18\x03 \x01(\x0b\x32,.nebius.logging.agentmanager.v1.ModuleHealthR\x0b\x63puPipeline\x12U\n\x0f\x63ilium_pipeline\x18\x04 \x01(\x0b\x32,.nebius.logging.agentmanager.v1.ModuleHealthR\x0e\x63iliumPipeline\x12U\n\x0fvmapps_pipeline\x18\x05 \x01(\x0b\x32,.nebius.logging.agentmanager.v1.ModuleHealthR\x0evmappsPipeline\"\xb7\x01\n\x0cModuleHealth\x12@\n\x05state\x18\x01 \x01(\x0e\x32*.nebius.logging.agentmanager.v1.AgentStateR\x05state\x12\x1a\n\x08messages\x18\x02 \x03(\tR\x08messages\x12I\n\nparameters\x18\x03 \x03(\x0b\x32).nebius.logging.agentmanager.v1.ParameterR\nparameters\"5\n\tParameter\x12\x12\n\x04name\x18\x01 \x01(\tR\x04name\x12\x14\n\x05value\x18\x02 \x01(\tR\x05value\"V\n\x06OSInfo\x12\x12\n\x04name\x18\x01 \x01(\tR\x04name\x12\x14\n\x05uname\x18\x02 \x01(\tR\x05uname\x12\"\n\x0c\x61rchitecture\x18\x03 \x01(\tR\x0c\x61rchitecture\"\xc4\x02\n\x12GetVersionResponse\x12>\n\x06\x61\x63tion\x18\x01 \x01(\x0e\x32&.nebius.logging.agentmanager.v1.ActionR\x06\x61\x63tion\x12\x43\n\x03nop\x18\x02 \x01(\x0b\x32/.nebius.logging.agentmanager.v1.NopActionParamsH\x00R\x03nop\x12L\n\x06update\x18\x03 \x01(\x0b\x32\x32.nebius.logging.agentmanager.v1.UpdateActionParamsH\x00R\x06update\x12O\n\x07restart\x18\x04 \x01(\x0b\x32\x33.nebius.logging.agentmanager.v1.RestartActionParamsH\x00R\x07restartB\n\n\x08response\"\x11\n\x0fNopActionParams\"I\n\x12UpdateActionParams\x12\x18\n\x07version\x18\x01 \x01(\tR\x07version\x12\x19\n\x08repo_url\x18\x02 \x01(\tR\x07repoUrl\"\x15\n\x13RestartActionParams*0\n\tAgentType\x12\x13\n\x0f\x41GENT_UNDEFINED\x10\x00\x12\x0e\n\nO11Y_AGENT\x10\x01*E\n\nAgentState\x12\x13\n\x0fSTATE_UNDEFINED\x10\x00\x12\x11\n\rSTATE_HEALTHY\x10\x01\x12\x0f\n\x0bSTATE_ERROR\x10\x02*@\n\x06\x41\x63tion\x12\x14\n\x10\x41\x43TION_UNDEFINED\x10\x00\x12\x07\n\x03NOP\x10\x01\x12\n\n\x06UPDATE\x10\x02\x12\x0b\n\x07RESTART\x10\x03\x32\xa5\x01\n\x0eVersionService\x12s\n\nGetVersion\x12\x31.nebius.logging.agentmanager.v1.GetVersionRequest\x1a\x32.nebius.logging.agentmanager.v1.GetVersionResponse\x1a\x1e\xbaJ\x1bobservability-agent-managerB|\n%ai.nebius.pub.logging.v1.agentmanagerB\x13VersionServiceProtoP\x01Z<github.com/nebius/gosdk/proto/nebius/logging/v1/agentmanagerb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'nebius.logging.v1.agentmanager.version_service_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n%ai.nebius.pub.logging.v1.agentmanagerB\023VersionServiceProtoP\001Z<github.com/nebius/gosdk/proto/nebius/logging/v1/agentmanager'
  _VERSIONSERVICE._options = None
  _VERSIONSERVICE._serialized_options = b'\272J\033observability-agent-manager'
  _globals['_AGENTTYPE']._serialized_start=2362
  _globals['_AGENTTYPE']._serialized_end=2410
  _globals['_AGENTSTATE']._serialized_start=2412
  _globals['_AGENTSTATE']._serialized_end=2481
  _globals['_ACTION']._serialized_start=2483
  _globals['_ACTION']._serialized_end=2547
  _globals['_GETVERSIONREQUEST']._serialized_start=147
  _globals['_GETVERSIONREQUEST']._serialized_end=1161
  _globals['_MODULESHEALTH']._serialized_start=1164
  _globals['_MODULESHEALTH']._serialized_end=1587
  _globals['_MODULEHEALTH']._serialized_start=1590
  _globals['_MODULEHEALTH']._serialized_end=1773
  _globals['_PARAMETER']._serialized_start=1775
  _globals['_PARAMETER']._serialized_end=1828
  _globals['_OSINFO']._serialized_start=1830
  _globals['_OSINFO']._serialized_end=1916
  _globals['_GETVERSIONRESPONSE']._serialized_start=1919
  _globals['_GETVERSIONRESPONSE']._serialized_end=2243
  _globals['_NOPACTIONPARAMS']._serialized_start=2245
  _globals['_NOPACTIONPARAMS']._serialized_end=2262
  _globals['_UPDATEACTIONPARAMS']._serialized_start=2264
  _globals['_UPDATEACTIONPARAMS']._serialized_end=2337
  _globals['_RESTARTACTIONPARAMS']._serialized_start=2339
  _globals['_RESTARTACTIONPARAMS']._serialized_end=2360
  _globals['_VERSIONSERVICE']._serialized_start=2550
  _globals['_VERSIONSERVICE']._serialized_end=2715
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: nebius/iam/v1/tenant_user_account_with_attributes_service.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from nebius.api.nebius import annotations_pb2 as nebius_dot_annotations__pb2
from nebius.api.nebius.iam.v1 import tenant_user_account_pb2 as nebius_dot_iam_dot_v1_dot_tenant__user__account__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n?nebius/iam/v1/tenant_user_account_with_attributes_service.proto\x12\rnebius.iam.v1\x1a\x18nebius/annotations.proto\x1a\'nebius/iam/v1/tenant_user_account.proto\";\n)GetTenantUserAccountWithAttributesRequest\x12\x0e\n\x02id\x18\x01 \x01(\tR\x02id\"\xb6\x01\n+ListTenantUserAccountsWithAttributesRequest\x12\x1b\n\tparent_id\x18\x01 \x01(\tR\x08parentId\x12 \n\tpage_size\x18\x02 \x01(\x03H\x00R\x08pageSize\x88\x01\x01\x12\x1d\n\npage_token\x18\x03 \x01(\tR\tpageToken\x12\x1b\n\x06\x66ilter\x18\x04 \x01(\tB\x03\xc0J\x01R\x06\x66ilterB\x0c\n\n_page_size\"\x9c\x01\n,ListTenantUserAccountsWithAttributesResponse\x12\x44\n\x05items\x18\x01 \x03(\x0b\x32..nebius.iam.v1.TenantUserAccountWithAttributesR\x05items\x12&\n\x0fnext_page_token\x18\x02 \x01(\tR\rnextPageToken2\xa6\x02\n&TenantUserAccountWithAttributesService\x12o\n\x03Get\x12\x38.nebius.iam.v1.GetTenantUserAccountWithAttributesRequest\x1a..nebius.iam.v1.TenantUserAccountWithAttributes\x12\x7f\n\x04List\x12:.nebius.iam.v1.ListTenantUserAccountsWithAttributesRequest\x1a;.nebius.iam.v1.ListTenantUserAccountsWithAttributesResponse\x1a\n\xbaJ\x07\x63pl.iamBr\n\x14\x61i.nebius.pub.iam.v1B+TenantUserAccountWithAttributesServiceProtoP\x01Z+github.com/nebius/gosdk/proto/nebius/iam/v1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'nebius.iam.v1.tenant_user_account_with_attributes_service_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\024ai.nebius.pub.iam.v1B+TenantUserAccountWithAttributesServiceProtoP\001Z+github.com/nebius/gosdk/proto/nebius/iam/v1'
  _LISTTENANTUSERACCOUNTSWITHATTRIBUTESREQUEST.fields_by_name['filter']._options = None
  _LISTTENANTUSERACCOUNTSWITHATTRIBUTESREQUEST.fields_by_name['filter']._serialized_options = b'\300J\001'
  _TENANTUSERACCOUNTWITHATTRIBUTESSERVICE._options = None
  _TENANTUSERACCOUNTWITHATTRIBUTESSERVICE._serialized_options = b'\272J\007cpl.iam'
  _globals['_GETTENANTUSERACCOUNTWITHATTRIBUTESREQUEST']._serialized_start=149
  _globals['_GETTENANTUSERACCOUNTWITHATTRIBUTESREQUEST']._serialized_end=208
  _globals['_LISTTENANTUSERACCOUNTSWITHATTRIBUTESREQUEST']._serialized_start=211
  _globals['_LISTTENANTUSERACCOUNTSWITHATTRIBUTESREQUEST']._serialized_end=393
  _globals['_LISTTENANTUSERACCOUNTSWITHATTRIBUTESRESPONSE']._serialized_start=396
  _globals['_LISTTENANTUSERACCOUNTSWITHATTRIBUTESRESPONSE']._serialized_end=552
  _globals['_TENANTUSERACCOUNTWITHATTRIBUTESSERVICE']._serialized_start=555
  _globals['_TENANTUSERACCOUNTWITHATTRIBUTESSERVICE']._serialized_end=849
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: nebius/compute/v1/platform_service.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from nebius.api.buf.validate import validate_pb2 as buf_dot_validate_dot_validate__pb2
from nebius.api.nebius import annotations_pb2 as nebius_dot_annotations__pb2
from nebius.api.nebius.common.v1 import metadata_pb2 as nebius_dot_common_dot_v1_dot_metadata__pb2
from nebius.api.nebius.compute.v1 import platform_pb2 as nebius_dot_compute_dot_v1_dot_platform__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n(nebius/compute/v1/platform_service.proto\x12\x11nebius.compute.v1\x1a\x1b\x62uf/validate/validate.proto\x1a\x18nebius/annotations.proto\x1a\x1fnebius/common/v1/metadata.proto\x1a nebius/compute/v1/platform.proto\"w\n\x14ListPlatformsRequest\x12\x1b\n\tpage_size\x18\x01 \x01(\x03R\x08pageSize\x12\x1d\n\npage_token\x18\x02 \x01(\tR\tpageToken\x12#\n\tparent_id\x18\x03 \x01(\tB\x06\xbaH\x03\xc8\x01\x01R\x08parentId\"r\n\x15ListPlatformsResponse\x12\x31\n\x05items\x18\x01 \x03(\x0b\x32\x1b.nebius.compute.v1.PlatformR\x05items\x12&\n\x0fnext_page_token\x18\x02 \x01(\tR\rnextPageToken2\xc6\x01\n\x0fPlatformService\x12L\n\tGetByName\x12\".nebius.common.v1.GetByNameRequest\x1a\x1b.nebius.compute.v1.Platform\x12Y\n\x04List\x12\'.nebius.compute.v1.ListPlatformsRequest\x1a(.nebius.compute.v1.ListPlatformsResponse\x1a\n\xbaJ\x07\x63omputeBc\n\x18\x61i.nebius.pub.compute.v1B\x14PlatformServiceProtoP\x01Z/github.com/nebius/gosdk/proto/nebius/compute/v1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'nebius.compute.v1.platform_service_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\030ai.nebius.pub.compute.v1B\024PlatformServiceProtoP\001Z/github.com/nebius/gosdk/proto/nebius/compute/v1'
  _LISTPLATFORMSREQUEST.fields_by_name['parent_id']._options = None
  _LISTPLATFORMSREQUEST.fields_by_name['parent_id']._serialized_options = b'\272H\003\310\001\001'
  _PLATFORMSERVICE._options = None
  _PLATFORMSERVICE._serialized_options = b'\272J\007compute'
  _globals['_LISTPLATFORMSREQUEST']._serialized_start=185
  _globals['_LISTPLATFORMSREQUEST']._serialized_end=304
  _globals['_LISTPLATFORMSRESPONSE']._serialized_start=306
  _globals['_LISTPLATFORMSRESPONSE']._serialized_end=420
  _globals['_PLATFORMSERVICE']._serialized_start=423
  _globals['_PLATFORMSERVICE']._serialized_end=621
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: nebius/iam/v1/group.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from nebius.api.buf.validate import validate_pb2 as buf_dot_validate_dot_validate__pb2
from nebius.api.nebius import annotations_pb2 as nebius_dot_annotations__pb2
from nebius.api.nebius.common.v1 import metadata_pb2 as nebius_dot_common_dot_v1_dot_metadata__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x19nebius/iam/v1/group.proto\x12\rnebius.iam.v1\x1a\x1b\x62uf/validate/validate.proto\x1a\x18nebius/annotations.proto\x1a\x1fnebius/common/v1/metadata.proto\"\xbf\x01\n\x05Group\x12\x46\n\x08metadata\x18\x01 \x01(\x0b\x32\".nebius.common.v1.ResourceMetadataB\x06\xbaH\x03\xc8\x01\x01R\x08metadata\x12\x34\n\x04spec\x18\x02 \x01(\x0b\x32\x18.nebius.iam.v1.GroupSpecB\x06\xbaH\x03\xc8\x01\x01R\x04spec\x12\x38\n\x06status\x18\x03 \x01(\x0b\x32\x1a.nebius.iam.v1.GroupStatusB\x04\xbaJ\x01\x05R\x06status\"\x0b\n\tGroupSpec\"\x83\x02\n\x0bGroupStatus\x12\x36\n\x05state\x18\x01 \x01(\x0e\x32 .nebius.iam.v1.GroupStatus.StateR\x05state\x12#\n\rmembers_count\x18\x02 \x01(\x05R\x0cmembersCount\x12\x34\n\x16service_accounts_count\x18\x03 \x01(\x05R\x14serviceAccountsCount\x12;\n\x1atenant_user_accounts_count\x18\x04 \x01(\x05R\x17tenantUserAccountsCount\"$\n\x05State\x12\x0f\n\x0bUNSPECIFIED\x10\x00\x12\n\n\x06\x41\x43TIVE\x10\x01\x42Q\n\x14\x61i.nebius.pub.iam.v1B\nGroupProtoP\x01Z+github.com/nebius/gosdk/proto/nebius/iam/v1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'nebius.iam.v1.group_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\024ai.nebius.pub.iam.v1B\nGroupProtoP\001Z+github.com/nebius/gosdk/proto/nebius/iam/v1'
  _GROUP.fields_by_name['metadata']._options = None
  _GROUP.fields_by_name['metadata']._serialized_options = b'\272H\003\310\001\001'
  _GROUP.fields_by_name['spec']._options = None
  _GROUP.fields_by_name['spec']._serialized_options = b'\272H\003\310\001\001'
  _GROUP.fields_by_name['status']._options = None
  _GROUP.fields_by_name['status']._serialized_options = b'\272J\001\005'
  _globals['_GROUP']._serialized_start=133
  _globals['_GROUP']._serialized_end=324
  _globals['_GROUPSPEC']._serialized_start=326
  _globals['_GROUPSPEC']._serialized_end=337
  _globals['_GROUPSTATUS']._serialized_start=340
  _globals['_GROUPSTATUS']._serialized_end=599
  _globals['_GROUPSTATUS_STATE']._serialized_start=563
  _globals['_GROUPSTATUS_STATE']._serialized_end=599
# @@protoc_insertion_point(module_scope)



from nebius.api.nebius import annotations_pb2 as _annotations_pb2
from nebius.api.nebius.iam.v1 import token_service_pb2 as _token_service_pb2
from google.protobuf import descriptor as _descriptor
from typing import ClassVar as _ClassVar

DESCRIPTOR: _descriptor.FileDescriptor



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: nebius/compute/v1/gpu_cluster_service.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from nebius.api.nebius import annotations_pb2 as nebius_dot_annotations__pb2
from nebius.api.nebius.common.v1 import metadata_pb2 as nebius_dot_common_dot_v1_dot_metadata__pb2
from nebius.api.nebius.common.v1 import operation_pb2 as nebius_dot_common_dot_v1_dot_operation__pb2
from nebius.api.nebius.common.v1 import operation_service_pb2 as nebius_dot_common_dot_v1_dot_operation__service__pb2
from nebius.api.nebius.compute.v1 import gpu_cluster_pb2 as nebius_dot_compute_dot_v1_dot_gpu__cluster__pb2
from nebius.api.nebius.compute.v1 import operation_service_pb2 as nebius_dot_compute_dot_v1_dot_operation__service__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n+nebius/compute/v1/gpu_cluster_service.proto\x12\x11nebius.compute.v1\x1a\x18nebius/annotations.proto\x1a\x1fnebius/common/v1/metadata.proto\x1a nebius/common/v1/operation.proto\x1a(nebius/common/v1/operation_service.proto\x1a#nebius/compute/v1/gpu_cluster.proto\x1a)nebius/compute/v1/operation_service.proto\"&\n\x14GetGpuClusterRequest\x12\x0e\n\x02id\x18\x01 \x01(\tR\x02id\"\xdb\x01\n\x16ListGpuClustersRequest\x12\x1b\n\tparent_id\x18\x01 \x01(\tR\x08parentId\x12\x1b\n\tpage_size\x18\x02 \x01(\x03R\x08pageSize\x12\x1d\n\npage_token\x18\x03 \x01(\tR\tpageToken\x12h\n\x06\x66ilter\x18\x04 \x01(\tBP\x18\x01\xd2JK\n\n2025-06-16\x12=it is not implemented, filtering could be done on client sideR\x06\x66ilter\"\x90\x01\n\x17\x43reateGpuClusterRequest\x12>\n\x08metadata\x18\x01 \x01(\x0b\x32\".nebius.common.v1.ResourceMetadataR\x08metadata\x12\x35\n\x04spec\x18\x02 \x01(\x0b\x32!.nebius.compute.v1.GpuClusterSpecR\x04spec\"\x90\x01\n\x17UpdateGpuClusterRequest\x12>\n\x08metadata\x18\x01 \x01(\x0b\x32\".nebius.common.v1.ResourceMetadataR\x08metadata\x12\x35\n\x04spec\x18\x02 \x01(\x0b\x32!.nebius.compute.v1.GpuClusterSpecR\x04spec\")\n\x17\x44\x65leteGpuClusterRequest\x12\x0e\n\x02id\x18\x01 \x01(\tR\x02id\"v\n\x17ListGpuClustersResponse\x12\x33\n\x05items\x18\x01 \x03(\x0b\x32\x1d.nebius.compute.v1.GpuClusterR\x05items\x12&\n\x0fnext_page_token\x18\x02 \x01(\tR\rnextPageToken2\x8c\x05\n\x11GpuClusterService\x12M\n\x03Get\x12\'.nebius.compute.v1.GetGpuClusterRequest\x1a\x1d.nebius.compute.v1.GpuCluster\x12N\n\tGetByName\x12\".nebius.common.v1.GetByNameRequest\x1a\x1d.nebius.compute.v1.GpuCluster\x12]\n\x04List\x12).nebius.compute.v1.ListGpuClustersRequest\x1a*.nebius.compute.v1.ListGpuClustersResponse\x12Q\n\x06\x43reate\x12*.nebius.compute.v1.CreateGpuClusterRequest\x1a\x1b.nebius.common.v1.Operation\x12Q\n\x06Update\x12*.nebius.compute.v1.UpdateGpuClusterRequest\x1a\x1b.nebius.common.v1.Operation\x12Q\n\x06\x44\x65lete\x12*.nebius.compute.v1.DeleteGpuClusterRequest\x1a\x1b.nebius.common.v1.Operation\x12t\n\x16ListOperationsByParent\x12\x30.nebius.compute.v1.ListOperationsByParentRequest\x1a(.nebius.common.v1.ListOperationsResponse\x1a\n\xbaJ\x07\x63omputeBe\n\x18\x61i.nebius.pub.compute.v1B\x16GpuClusterServiceProtoP\x01Z/github.com/nebius/gosdk/proto/nebius/compute/v1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'nebius.compute.v1.gpu_cluster_service_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\030ai.nebius.pub.compute.v1B\026GpuClusterServiceProtoP\001Z/github.com/nebius/gosdk/proto/nebius/compute/v1'
  _LISTGPUCLUSTERSREQUEST.fields_by_name['filter']._options = None
  _LISTGPUCLUSTERSREQUEST.fields_by_name['filter']._serialized_options = b'\030\001\322JK\n\n2025-06-16\022=it is not implemented, filtering could be done on client side'
  _GPUCLUSTERSERVICE._options = None
  _GPUCLUSTERSERVICE._serialized_options = b'\272J\007compute'
  _globals['_GETGPUCLUSTERREQUEST']._serialized_start=281
  _globals['_GETGPUCLUSTERREQUEST']._serialized_end=319
  _globals['_LISTGPUCLUSTERSREQUEST']._serialized_start=322
  _globals['_LISTGPUCLUSTERSREQUEST']._serialized_end=541
  _globals['_CREATEGPUCLUSTERREQUEST']._serialized_start=544
  _globals['_CREATEGPUCLUSTERREQUEST']._serialized_end=688
  _globals['_UPDATEGPUCLUSTERREQUEST']._serialized_start=691
  _globals['_UPDATEGPUCLUSTERREQUEST']._serialized_end=835
  _globals['_DELETEGPUCLUSTERREQUEST']._serialized_start=837
  _globals['_DELETEGPUCLUSTERREQUEST']._serialized_end=878
  _globals['_LISTGPUCLUSTERSRESPONSE']._serialized_start=880
  _globals['_LISTGPUCLUSTERSRESPONSE']._serialized_end=998
  _globals['_GPUCLUSTERSERVICE']._serialized_start=1001
  _globals['_GPUCLUSTERSERVICE']._serialized_end=1653
# @@protoc_insertion_point(module_scope)



"""CLI-style configuration reader used by the SDK.

This module provides a small :class:`Config` helper to read Nebius CLI-style
configuration files and translate profile entries into credential bearers
that the SDK can use. It supports multiple auth types such as federation and
service-account credentials and will prefer an environment-supplied token if
present.

The primary entrypoint is :class:`Config.get_credentials` which returns a
credentials object ready to be consumed by :class:`nebius.aio.channel.Channel`.
"""

from logging import getLogger
from os import environ
from os.path import isfile
from pathlib import Path
from ssl import SSLContext
from typing import Any, TextIO

from nebius.aio.abc import ClientChannelInterface
from nebius.aio.authorization.authorization import Provider as AuthorizationProvider
from nebius.aio.token.service_account import ServiceAccountBearer
from nebius.aio.token.static import EnvBearer, NoTokenInEnvError
from nebius.aio.token.token import Bearer as TokenBearer
from nebius.aio.token.token import Token
from nebius.base.constants import (
    DEFAULT_CONFIG_DIR,
    DEFAULT_CONFIG_FILE,
    ENDPOINT_ENV,
    PROFILE_ENV,
    TOKEN_ENV,
)
from nebius.base.error import SDKError
from nebius.base.service_account.service_account import (
    TokenRequester as TokenRequestReader,
)

Credentials = AuthorizationProvider | TokenBearer | TokenRequestReader | Token | str


log = getLogger(__name__)


class ConfigError(SDKError):
    """Base exception for configuration-related errors."""


class NoParentIdError(ConfigError):
    """Raised when a requested parent id is missing or explicitly disabled."""


class Config:
    """Reader for Nebius CLI-style configuration files.

    The :class:`Config` class locates and parses a YAML-based configuration
    file (by default under ``~/.nebius/config.yaml``) and exposes convenience
    methods to obtain the default parent id, endpoint, and credentials
    configured for the active profile.

    :param client_id: Optional client id used for federation flows.
    :type client_id: optional `str`
    :param config_file: Path to the configuration YAML file.
    :type config_file: `str` | `Path`
    :param profile: Optional profile name to select; when omitted the
        default profile from the config or the environment variable
        indicated by ``profile_env`` is used.
    :type profile: optional `str`
    :param profile_env: Environment variable name used to select a profile.
    :type profile_env: `str`
    :param token_env: Environment variable name that may contain an IAM token
        and will take priority over file-based credentials.
    :type token_env: `str`
    :param no_env: If True skip environment token lookup, profile selection, and
        endpoint override. If you want to disable only one of these features, you can
        set the env variable name to some invalid value.
    :type no_env: `bool`
    :param no_parent_id: If True disable automatic parent id resolution.
    :type no_parent_id: `bool`
    :param max_retries: Maximum number of auth retries when interacting with
        external services (passed to underlying bearers).
    :param endpoint: Optional endpoint URL to override profile setting.
    :type endpoint: optional `str`
    :param endpoint_env: Environment variable name used to override the
        endpoint URL from the profile.
    :type endpoint_env: `str`
    :type max_retries: `int`

    Example
    -------

    Initialize the SDK with CLI config::

        from nebius.sdk import SDK
        from nebius.aio.cli_config import Config

        # Initialize SDK with CLI config reader
        sdk = SDK(config_reader=Config())

        # The config reader will automatically handle authentication
        # based on the active CLI profile

        # You can also access config properties directly
        config = Config()
        print(f"Default parent ID: {config.parent_id}")
        print(f"Endpoint: {config.endpoint()}")
    """

    def __init__(
        self,
        client_id: str | None = None,
        config_file: str | Path = Path(DEFAULT_CONFIG_DIR) / DEFAULT_CONFIG_FILE,
        profile: str | None = None,
        profile_env: str = PROFILE_ENV,
        token_env: str = TOKEN_ENV,
        no_env: bool = False,
        no_parent_id: bool = False,
        max_retries: int = 2,
        endpoint: str | None = None,
        endpoint_env: str = ENDPOINT_ENV,
    ) -> None:
        """Initialize the config reader, and read the config file, selecting
        the active profile.
        """
        self._client_id = client_id
        self._priority_bearer: EnvBearer | None = None
        self._profile_name = profile
        self._endpoint: str | None = endpoint
        if not no_env:
            try:
                self._priority_bearer = EnvBearer(env_var_name=token_env)
            except NoTokenInEnvError:
                pass
            if self._profile_name is None:
                self._profile_name = environ.get(profile_env, None)
            if self._endpoint is None:
                self._endpoint = environ.get(endpoint_env, None)
        self._no_parent_id = no_parent_id
        self._config_file = Path(config_file).expanduser()
        self._max_retries = max_retries
        self._get_profile()

    @property
    def parent_id(self) -> str:
        """Return the parent id from the active profile.

        The value is read from the active profile's ``parent-id`` field and
        validated to be a non-empty string.

        :returns: the parent id configured for the active profile
        :rtype: `str`
        :raises NoParentIdError: if parent id usage is disabled or the value is
            missing or empty in the profile
        :raises ConfigError: if the profile contains a non-string parent-id
        """
        if self._no_parent_id:
            raise NoParentIdError(
                "Config is set to not use parent id from the profile."
            )
        if "parent-id" not in self._profile:
            raise NoParentIdError("Missing parent-id in the profile.")
        if not isinstance(self._profile["parent-id"], str):
            raise ConfigError(
                "Parent id should be a string, got "
                f"{type(self._profile['parent-id'])}."
            )
        if self._profile["parent-id"] == "":
            raise NoParentIdError("Parent id is empty.")

        return self._profile["parent-id"]

    def endpoint(self) -> str:
        """Return the configured endpoint for the active profile.

        If the profile does not define an endpoint this method returns an
        empty string.

        :returns: endpoint string or empty string when not configured
        :rtype: `str`
        """
        return self._endpoint or ""

    def _get_profile(self) -> None:
        """Get the profile from the config file."""
        import yaml

        if not isfile(self._config_file):
            raise FileNotFoundError(f"Config file {self._config_file} not found.")

        with open(self._config_file, "r") as f:
            config = yaml.safe_load(f)

        if "profiles" not in config:
            raise ConfigError("No profiles found in the config file.")
        if not isinstance(config["profiles"], dict):
            raise ConfigError(
                f"Profiles should be a dictionary, got {type(config['profiles'])}."
            )
        if len(config["profiles"]) == 0:
            raise ConfigError(
                "No profiles found in the config file, setup the nebius CLI profile"
                " first."
            )
        if self._profile_name is None:
            if "default" not in config:
                if len(config["profiles"]) == 1:
                    self._profile_name = list(config["profiles"].keys())[0]
                else:
                    raise ConfigError("No default profile found in the config file.")
            else:
                self._profile_name = config["default"]
            if self._profile_name is None:
                raise ConfigError(
                    "No profile selected. Either set the profile in the config setup,"
                    " set the env var NEBIUS_PROFILE or "
                    "execute `nebius profile activate`."
                )
        profile = self._profile_name
        if not isinstance(profile, str):
            raise ConfigError(f"Profile name should be a string, got {type(profile)}.")
        if profile not in config["profiles"]:
            raise ConfigError(f"Profile {profile} not found in the config file.")
        if not isinstance(config["profiles"][profile], dict):
            raise ConfigError(
                f"Profile {profile} should be a dictionary, got "
                f"{type(config['profiles'][profile])}."
            )
        self._profile: dict[str, Any] = config["profiles"][profile]

        if (
            self._endpoint is None
            or self._endpoint == ""
            and "endpoint" in self._profile
        ):
            if not isinstance(self._profile["endpoint"], str):
                raise ConfigError(
                    "Endpoint should be a string, got "
                    f"{type(self._profile['endpoint'])}."
                )
            self._endpoint = self._profile["endpoint"]

    def get_credentials(
        self,
        channel: ClientChannelInterface,
        writer: TextIO | None = None,
        no_browser_open: bool = False,
        ssl_ctx: SSLContext | None = None,
    ) -> Credentials:
        """Resolve and return credentials for the active profile.

        This method consults, in order of priority:

        1. An environment-provided token bearer (if present and enabled).
        2. A token file specified by ``token-file`` in the profile.
        3. The profile's ``auth-type`` which may be ``federation`` or
           ``service account`` and will create the corresponding bearer
           implementation.

        The returned object is suitable to be consumed by
        :class:`nebius.aio.channel.Channel` and may be one of
        :class:`nebius.aio.authorization.authorization.Provider`, a
        :class:`nebius.aio.token.token.Bearer`, a :class:`TokenRequester`
        reader, a low-level :class:`nebius.aio.token.token.Token`, or a raw
        string token depending on the profile and environment.

        :param channel: channel instance used for network-bound credential flows
        :type channel: :class:`ClientChannelInterface`
        :param writer: optional text stream used by interactive flows (federation)
        :type writer: optional `TextIO`
        :param no_browser_open: when True, federation flows will not open browsers
        :type no_browser_open: `bool`
        :param ssl_ctx: optional SSLContext forwarded to federation flows
        :type ssl_ctx: optional `SSLContext`

        :returns: a credentials object appropriate for the active profile
        :rtype: :class:`Provider`, :class:`nebius.aio.token.token.Bearer`,
            :class:`TokenRequester`, :class:`Token` or `str`
        :raises ConfigError: for malformed or missing profile entries
        """
        if self._priority_bearer is not None:
            return self._priority_bearer
        if "token-file" in self._profile:
            from nebius.aio.token.file import Bearer as FileBearer

            if not isinstance(self._profile["token-file"], str):
                raise ConfigError(
                    "Token file should be a string, got "
                    f" {type(self._profile['token-file'])}."
                )
            return FileBearer(self._profile["token-file"])
        if "auth-type" not in self._profile:
            raise ConfigError("Missing auth-type in the profile.")
        auth_type = self._profile["auth-type"]
        if auth_type == "federation":
            if "federation-endpoint" not in self._profile:
                raise ConfigError("Missing federation-endpoint in the profile.")
            if not isinstance(self._profile["federation-endpoint"], str):
                raise ConfigError(
                    "Federation endpoint should be a string, got "
                    f"{type(self._profile['federation-endpoint'])}."
                )
            if "federation-id" not in self._profile:
                raise ConfigError("Missing federation-id in the profile.")
            if not isinstance(self._profile["federation-id"], str):
                raise ConfigError(
                    "Federation id should be a string, got "
                    f"{type(self._profile['federation-id'])}."
                )
            from nebius.aio.token.federation_account import FederationBearer

            if not self._client_id:
                raise ConfigError(
                    "Client ID is required for federation authentication."
                )

            log.debug(
                f"Creating FederationBearer with profile {self._profile_name}, "
                f"client_id {self._client_id}, "
                f"federation_url {self._profile['federation-endpoint']}, "
                f"federation_id {self._profile['federation-id']}, "
                f"writer {writer}, no_browser_open {no_browser_open}."
            )

            # Federation bearer already wraps underlying signer with a file-cache
            # renewable bearer. Return it directly.
            return FederationBearer(
                profile_name=self._profile_name,  # type: ignore
                client_id=self._client_id,
                federation_endpoint=self._profile["federation-endpoint"],
                federation_id=self._profile["federation-id"],
                writer=writer,
                no_browser_open=no_browser_open,
                ssl_ctx=ssl_ctx,
            )
        elif auth_type == "service account":
            from cryptography.hazmat.backends import default_backend
            from cryptography.hazmat.primitives import serialization
            from cryptography.hazmat.primitives.asymmetric.rsa import RSAPrivateKey

            # Possible sources (priority):
            # 1) federated-subject-credentials-file-path + service-account-id
            # 2) service-account-credentials-file-path
            # 3) inline private-key with service-account-id + public-key-id
            # 4) private-key-file-path with service-account-id + public-key-id

            svc_id: str | None = None
            if "service-account-id" in self._profile:
                if not isinstance(self._profile["service-account-id"], str):
                    raise ConfigError(
                        "Service account should be a string, got "
                        f"{type(self._profile['service-account-id'])}."
                    )
                svc_id = self._profile["service-account-id"]

            # 1) federated subject credentials file
            if (
                svc_id is not None
                and "federated-subject-credentials-file-path" in self._profile
            ):
                if not isinstance(
                    self._profile["federated-subject-credentials-file-path"], str
                ):
                    raise ConfigError(
                        "federated-subject-credentials-file-path should be a string"
                    )
                # Use the federated credentials bearer (file-backed) to supply
                # the actor token for token-exchange. Do not treat this as a
                # service-account credentials file.
                from nebius.aio.token.federated_credentials import (
                    FederatedCredentialsBearer,
                )

                return FederatedCredentialsBearer(
                    self._profile["federated-subject-credentials-file-path"],
                    service_account_id=svc_id,
                    channel=channel,
                )

            # 2) service account credentials file
            if "service-account-credentials-file-path" in self._profile:
                if not isinstance(
                    self._profile["service-account-credentials-file-path"], str
                ):
                    raise ConfigError(
                        "service-account-credentials-file-path should be a string"
                    )
                from nebius.base.service_account.credentials_file import (
                    Reader as CredentialsFileReader,
                )

                return ServiceAccountBearer(
                    service_account=CredentialsFileReader(
                        self._profile["service-account-credentials-file-path"]
                    ),
                    channel=channel,
                )

            # 3 & 4) inline private-key or private-key file path
            if svc_id is None:
                raise ConfigError("Missing service-account-id in the profile.")

            if "public-key-id" not in self._profile:
                raise ConfigError("Missing public-key-id in the profile.")
            if not isinstance(self._profile["public-key-id"], str):
                raise ConfigError(
                    "Public key should be a string, got "
                    f"{type(self._profile['public-key-id'])}."
                )
            pk_id = self._profile["public-key-id"]

            # inline private-key
            if "private-key" in self._profile:
                if not isinstance(self._profile["private-key"], str):
                    raise ConfigError(
                        "Private key should be a string, got "
                        f"{type(self._profile['private-key'])}."
                    )
                pk = serialization.load_pem_private_key(
                    self._profile["private-key"].encode("utf-8"),
                    password=None,
                    backend=default_backend(),
                )
                if not isinstance(pk, RSAPrivateKey):
                    raise ConfigError(
                        f"Private key should be of type RSAPrivateKey, got {type(pk)}."
                    )
                return ServiceAccountBearer(
                    service_account=svc_id,
                    public_key_id=pk_id,
                    private_key=pk,
                    channel=channel,
                )

            # private-key file path
            if "private-key-file-path" in self._profile:
                if not isinstance(self._profile["private-key-file-path"], str):
                    raise ConfigError("private-key-file-path should be a string")
                from nebius.base.service_account.pk_file import Reader as PKFileReader

                return ServiceAccountBearer(
                    service_account=PKFileReader(
                        self._profile["private-key-file-path"], pk_id, svc_id
                    ),
                    channel=channel,
                )

            # Nothing matched
            raise ConfigError(
                "Incomplete service account configuration: provide either "
                "(service-account-id and federated-subject-credentials-file-path) OR "
                "(service-account-credentials-file-path) OR "
                "(service-account-id, public-key-id and one of "
                "private-key / private-key-file-path)"
            )
        else:
            raise ConfigError(f"Unsupported auth-type {auth_type} in the profile.")



from nebius.api.buf.validate import validate_pb2 as _validate_pb2
from nebius.api.nebius.common.v1 import metadata_pb2 as _metadata_pb2
from nebius.api.nebius import annotations_pb2 as _annotations_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class Disk(_message.Message):
    __slots__ = ["metadata", "spec", "status"]
    METADATA_FIELD_NUMBER: _ClassVar[int]
    SPEC_FIELD_NUMBER: _ClassVar[int]
    STATUS_FIELD_NUMBER: _ClassVar[int]
    metadata: _metadata_pb2.ResourceMetadata
    spec: DiskSpec
    status: DiskStatus
    def __init__(self, metadata: _Optional[_Union[_metadata_pb2.ResourceMetadata, _Mapping]] = ..., spec: _Optional[_Union[DiskSpec, _Mapping]] = ..., status: _Optional[_Union[DiskStatus, _Mapping]] = ...) -> None: ...

class DiskSpec(_message.Message):
    __slots__ = ["size_bytes", "size_kibibytes", "size_mebibytes", "size_gibibytes", "block_size_bytes", "type", "source_image_id", "source_image_family", "disk_encryption"]
    class DiskType(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
        __slots__ = []
        UNSPECIFIED: _ClassVar[DiskSpec.DiskType]
        NETWORK_SSD: _ClassVar[DiskSpec.DiskType]
        NETWORK_HDD: _ClassVar[DiskSpec.DiskType]
        NETWORK_SSD_NON_REPLICATED: _ClassVar[DiskSpec.DiskType]
        NETWORK_SSD_IO_M3: _ClassVar[DiskSpec.DiskType]
    UNSPECIFIED: DiskSpec.DiskType
    NETWORK_SSD: DiskSpec.DiskType
    NETWORK_HDD: DiskSpec.DiskType
    NETWORK_SSD_NON_REPLICATED: DiskSpec.DiskType
    NETWORK_SSD_IO_M3: DiskSpec.DiskType
    SIZE_BYTES_FIELD_NUMBER: _ClassVar[int]
    SIZE_KIBIBYTES_FIELD_NUMBER: _ClassVar[int]
    SIZE_MEBIBYTES_FIELD_NUMBER: _ClassVar[int]
    SIZE_GIBIBYTES_FIELD_NUMBER: _ClassVar[int]
    BLOCK_SIZE_BYTES_FIELD_NUMBER: _ClassVar[int]
    TYPE_FIELD_NUMBER: _ClassVar[int]
    SOURCE_IMAGE_ID_FIELD_NUMBER: _ClassVar[int]
    SOURCE_IMAGE_FAMILY_FIELD_NUMBER: _ClassVar[int]
    DISK_ENCRYPTION_FIELD_NUMBER: _ClassVar[int]
    size_bytes: int
    size_kibibytes: int
    size_mebibytes: int
    size_gibibytes: int
    block_size_bytes: int
    type: DiskSpec.DiskType
    source_image_id: str
    source_image_family: SourceImageFamily
    disk_encryption: DiskEncryption
    def __init__(self, size_bytes: _Optional[int] = ..., size_kibibytes: _Optional[int] = ..., size_mebibytes: _Optional[int] = ..., size_gibibytes: _Optional[int] = ..., block_size_bytes: _Optional[int] = ..., type: _Optional[_Union[DiskSpec.DiskType, str]] = ..., source_image_id: _Optional[str] = ..., source_image_family: _Optional[_Union[SourceImageFamily, _Mapping]] = ..., disk_encryption: _Optional[_Union[DiskEncryption, _Mapping]] = ...) -> None: ...

class SourceImageFamily(_message.Message):
    __slots__ = ["image_family", "parent_id"]
    IMAGE_FAMILY_FIELD_NUMBER: _ClassVar[int]
    PARENT_ID_FIELD_NUMBER: _ClassVar[int]
    image_family: str
    parent_id: str
    def __init__(self, image_family: _Optional[str] = ..., parent_id: _Optional[str] = ...) -> None: ...

class DiskStatus(_message.Message):
    __slots__ = ["state", "state_description", "read_write_attachment", "read_only_attachments", "source_image_id", "size_bytes", "reconciling", "block_size_bytes", "source_image_cpu_architecture"]
    class State(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
        __slots__ = []
        UNSPECIFIED: _ClassVar[DiskStatus.State]
        CREATING: _ClassVar[DiskStatus.State]
        READY: _ClassVar[DiskStatus.State]
        UPDATING: _ClassVar[DiskStatus.State]
        DELETING: _ClassVar[DiskStatus.State]
        ERROR: _ClassVar[DiskStatus.State]
        BROKEN: _ClassVar[DiskStatus.State]
    UNSPECIFIED: DiskStatus.State
    CREATING: DiskStatus.State
    READY: DiskStatus.State
    UPDATING: DiskStatus.State
    DELETING: DiskStatus.State
    ERROR: DiskStatus.State
    BROKEN: DiskStatus.State
    class SourceImageCPUArchitecture(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
        __slots__ = []
        SOURCE_IMAGE_CPU_UNSPECIFIED: _ClassVar[DiskStatus.SourceImageCPUArchitecture]
        AMD64: _ClassVar[DiskStatus.SourceImageCPUArchitecture]
        ARM64: _ClassVar[DiskStatus.SourceImageCPUArchitecture]
    SOURCE_IMAGE_CPU_UNSPECIFIED: DiskStatus.SourceImageCPUArchitecture
    AMD64: DiskStatus.SourceImageCPUArchitecture
    ARM64: DiskStatus.SourceImageCPUArchitecture
    STATE_FIELD_NUMBER: _ClassVar[int]
    STATE_DESCRIPTION_FIELD_NUMBER: _ClassVar[int]
    READ_WRITE_ATTACHMENT_FIELD_NUMBER: _ClassVar[int]
    READ_ONLY_ATTACHMENTS_FIELD_NUMBER: _ClassVar[int]
    SOURCE_IMAGE_ID_FIELD_NUMBER: _ClassVar[int]
    SIZE_BYTES_FIELD_NUMBER: _ClassVar[int]
    RECONCILING_FIELD_NUMBER: _ClassVar[int]
    BLOCK_SIZE_BYTES_FIELD_NUMBER: _ClassVar[int]
    SOURCE_IMAGE_CPU_ARCHITECTURE_FIELD_NUMBER: _ClassVar[int]
    state: DiskStatus.State
    state_description: str
    read_write_attachment: str
    read_only_attachments: _containers.RepeatedScalarFieldContainer[str]
    source_image_id: str
    size_bytes: int
    reconciling: bool
    block_size_bytes: int
    source_image_cpu_architecture: DiskStatus.SourceImageCPUArchitecture
    def __init__(self, state: _Optional[_Union[DiskStatus.State, str]] = ..., state_description: _Optional[str] = ..., read_write_attachment: _Optional[str] = ..., read_only_attachments: _Optional[_Iterable[str]] = ..., source_image_id: _Optional[str] = ..., size_bytes: _Optional[int] = ..., reconciling: bool = ..., block_size_bytes: _Optional[int] = ..., source_image_cpu_architecture: _Optional[_Union[DiskStatus.SourceImageCPUArchitecture, str]] = ...) -> None: ...

class DiskEncryption(_message.Message):
    __slots__ = ["type"]
    class DiskEncryptionType(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
        __slots__ = []
        DISK_ENCRYPTION_UNSPECIFIED: _ClassVar[DiskEncryption.DiskEncryptionType]
        DISK_ENCRYPTION_MANAGED: _ClassVar[DiskEncryption.DiskEncryptionType]
    DISK_ENCRYPTION_UNSPECIFIED: DiskEncryption.DiskEncryptionType
    DISK_ENCRYPTION_MANAGED: DiskEncryption.DiskEncryptionType
    TYPE_FIELD_NUMBER: _ClassVar[int]
    type: DiskEncryption.DiskEncryptionType
    def __init__(self, type: _Optional[_Union[DiskEncryption.DiskEncryptionType, str]] = ...) -> None: ...



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from nebius.api.nebius.compute.v1 import node_service_pb2 as nebius_dot_compute_dot_v1_dot_node__service__pb2


class NodeServiceStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.SetUnhealthy = channel.unary_unary(
                '/nebius.compute.v1.NodeService/SetUnhealthy',
                request_serializer=nebius_dot_compute_dot_v1_dot_node__service__pb2.NodeSetUnhealthyRequest.SerializeToString,
                response_deserializer=nebius_dot_compute_dot_v1_dot_node__service__pb2.NodeSetUnhealthyResponse.FromString,
                )


class NodeServiceServicer(object):
    """Missing associated documentation comment in .proto file."""

    def SetUnhealthy(self, request, context):
        """SetUnhealthy marks the node underlying the Compute VM as unhealthy, which has the following effect:

        1. Scheduler makes the best effort not to assign new VMs to the unhealthy node,
        but in case of no capacity, the VM can be assigned to an unhealthy node.
        2. The existing VMs continue to work on the node, but after stop/start via
        Compute API they most probably will be assigned to a different node.

        To use this rpc one needs to obtain `compute.node.setUnhealthy` permission
        for the VM's parent container. The permission is granted to the TSA inside the VM.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_NodeServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'SetUnhealthy': grpc.unary_unary_rpc_method_handler(
                    servicer.SetUnhealthy,
                    request_deserializer=nebius_dot_compute_dot_v1_dot_node__service__pb2.NodeSetUnhealthyRequest.FromString,
                    response_serializer=nebius_dot_compute_dot_v1_dot_node__service__pb2.NodeSetUnhealthyResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'nebius.compute.v1.NodeService', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class NodeService(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def SetUnhealthy(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nebius.compute.v1.NodeService/SetUnhealthy',
            nebius_dot_compute_dot_v1_dot_node__service__pb2.NodeSetUnhealthyRequest.SerializeToString,
            nebius_dot_compute_dot_v1_dot_node__service__pb2.NodeSetUnhealthyResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



from typing import Any, Protocol, TypeVar

from google.protobuf.message import Message
from grpc import (
    CallCredentials,
    ChannelConnectivity,
    Compression,
)
from grpc.aio import Channel as GRPCChannel
from grpc.aio._base_call import (
    StreamStreamCall,
    StreamUnaryCall,
    UnaryStreamCall,
    UnaryUnaryCall,
)
from grpc.aio._base_channel import (
    StreamStreamMultiCallable,
    StreamUnaryMultiCallable,
    UnaryStreamMultiCallable,
    UnaryUnaryMultiCallable,
)
from grpc.aio._typing import (
    DeserializingFunction,
    MetadataType,
    RequestIterableType,
    SerializingFunction,
)

from nebius.base.error import SDKError
from nebius.base.methods import service_from_method_name

Req = TypeVar("Req", bound=Message)
Res = TypeVar("Res", bound=Message)


class NotATrueCallError(SDKError):
    def __init__(self, *args: object) -> None:
        super().__init__("This class is not meant to be run as a call.")


class NoMethodsInServiceError(SDKError):
    def __init__(self, *args: object) -> None:
        super().__init__("Mo methods found in service stub")


class StubUU(UnaryUnaryMultiCallable):  # type: ignore[unused-ignore,misc,type-arg]
    def __call__(  # type: ignore
        self,
        request,  # type: ignore[unused-ignore]
        *,
        timeout: float | None = None,
        metadata: MetadataType | None = None,
        credentials: CallCredentials | None = None,
        wait_for_ready: bool | None = None,
        compression: Compression | None = None,
    ) -> UnaryUnaryCall:  # type: ignore[unused-ignore, type-arg]
        raise NotATrueCallError()


class StubUS(UnaryStreamMultiCallable):  # type: ignore[unused-ignore,misc,type-arg]
    def __call__(  # type: ignore
        self,
        request,  # type: ignore[unused-ignore]
        *,
        timeout: float | None = None,
        metadata: MetadataType | None = None,
        credentials: CallCredentials | None = None,
        wait_for_ready: bool | None = None,
        compression: Compression | None = None,
    ) -> UnaryStreamCall:  # type: ignore[unused-ignore, type-arg]
        raise NotATrueCallError()


class StubSU(StreamUnaryMultiCallable):  # type: ignore[unused-ignore,misc]
    def __call__(  # type: ignore[unused-ignore]
        self,
        request_iterator: RequestIterableType | None = None,
        timeout: float | None = None,
        metadata: MetadataType | None = None,
        credentials: CallCredentials | None = None,
        wait_for_ready: bool | None = None,
        compression: Compression | None = None,
    ) -> StreamUnaryCall:  # type: ignore[unused-ignore, type-arg]
        raise NotATrueCallError()


class StubSS(StreamStreamMultiCallable):  # type: ignore[unused-ignore,misc]
    def __call__(  # type: ignore[unused-ignore]
        self,
        request_iterator: RequestIterableType | None = None,
        timeout: float | None = None,
        metadata: MetadataType | None = None,
        credentials: CallCredentials | None = None,
        wait_for_ready: bool | None = None,
        compression: Compression | None = None,
    ) -> StreamStreamCall:  # type: ignore[unused-ignore, type-arg]
        raise NotATrueCallError()


class ExtractorChannel(GRPCChannel):  # type: ignore[unused-ignore,misc]
    def __init__(self) -> None:
        super().__init__()
        self._last_method = ""

    def get_service_name(self) -> str:
        if self._last_method == "":
            raise NoMethodsInServiceError()
        return service_from_method_name(self._last_method)

    def unary_unary(  # type: ignore[unused-ignore, override]
        self,
        method: str,
        request_serializer: SerializingFunction | None = None,
        response_deserializer: DeserializingFunction | None = None,
        _registered_method: bool | None = False,
    ) -> UnaryUnaryMultiCallable[Req, Res]:  # type: ignore[unused-ignore, override]
        self._last_method = method
        return StubUU()

    async def close(self, grace: float | None = None) -> None:
        pass

    async def __aenter__(self) -> "ExtractorChannel":
        return self

    async def __aexit__(self, exc_type: Any, exc_val: Any, exc_tb: Any) -> None:
        await self.close(None)

    def get_state(self, try_to_connect: bool = False) -> ChannelConnectivity:
        return ChannelConnectivity.READY

    async def wait_for_state_change(
        self,
        last_observed_state: ChannelConnectivity,
    ) -> None:
        raise NotImplementedError("this method has no meaning for this channel")

    async def channel_ready(self) -> None:
        return

    def unary_stream(  # type: ignore[override]
        self,
        method: str,
        request_serializer: SerializingFunction | None = None,
        response_deserializer: DeserializingFunction | None = None,
        _registered_method: bool | None = None,
    ) -> UnaryStreamMultiCallable[Req, Res]:  # type: ignore[unused-ignore]
        self._last_method = method
        raise StubUS()  # type: ignore[misc]

    def stream_unary(  # type: ignore[override]
        self,
        method: str,
        request_serializer: SerializingFunction | None = None,
        response_deserializer: DeserializingFunction | None = None,
        _registered_method: bool | None = None,
    ) -> StreamUnaryMultiCallable:
        self._last_method = method
        raise StubSU()  # type: ignore[misc]

    def stream_stream(  # type: ignore[override]
        self,
        method: str,
        request_serializer: SerializingFunction | None = None,
        response_deserializer: DeserializingFunction | None = None,
        _registered_method: bool | None = None,
    ) -> StreamStreamMultiCallable:
        self._last_method = method
        raise StubSS()  # type: ignore[misc]


class ServiceStub(Protocol):
    def __init__(self, channel: GRPCChannel) -> None: ...


def from_stub_class(stub: type[ServiceStub]) -> str:
    if hasattr(stub, "__PB2_NAME__"):
        return getattr(stub, "__PB2_NAME__")  # type: ignore[no-any-return]
    extractor = ExtractorChannel()
    _ = stub(extractor)
    ret = extractor.get_service_name()
    setattr(stub, "__PB2_NAME__", ret)
    return ret



from nebius.api.buf.validate import validate_pb2 as _validate_pb2
from nebius.api.nebius import annotations_pb2 as _annotations_pb2
from nebius.api.nebius.common.v1 import metadata_pb2 as _metadata_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class Zone(_message.Message):
    __slots__ = ["metadata", "spec", "status"]
    METADATA_FIELD_NUMBER: _ClassVar[int]
    SPEC_FIELD_NUMBER: _ClassVar[int]
    STATUS_FIELD_NUMBER: _ClassVar[int]
    metadata: _metadata_pb2.ResourceMetadata
    spec: ZoneSpec
    status: ZoneStatus
    def __init__(self, metadata: _Optional[_Union[_metadata_pb2.ResourceMetadata, _Mapping]] = ..., spec: _Optional[_Union[ZoneSpec, _Mapping]] = ..., status: _Optional[_Union[ZoneStatus, _Mapping]] = ...) -> None: ...

class ZoneSpec(_message.Message):
    __slots__ = ["domain_name", "vpc", "soa_spec"]
    DOMAIN_NAME_FIELD_NUMBER: _ClassVar[int]
    VPC_FIELD_NUMBER: _ClassVar[int]
    SOA_SPEC_FIELD_NUMBER: _ClassVar[int]
    domain_name: str
    vpc: VpcZoneScope
    soa_spec: SoaSpec
    def __init__(self, domain_name: _Optional[str] = ..., vpc: _Optional[_Union[VpcZoneScope, _Mapping]] = ..., soa_spec: _Optional[_Union[SoaSpec, _Mapping]] = ...) -> None: ...

class VpcZoneScope(_message.Message):
    __slots__ = ["primary_network_id"]
    PRIMARY_NETWORK_ID_FIELD_NUMBER: _ClassVar[int]
    primary_network_id: str
    def __init__(self, primary_network_id: _Optional[str] = ...) -> None: ...

class SoaSpec(_message.Message):
    __slots__ = ["negative_ttl"]
    NEGATIVE_TTL_FIELD_NUMBER: _ClassVar[int]
    negative_ttl: int
    def __init__(self, negative_ttl: _Optional[int] = ...) -> None: ...

class ZoneStatus(_message.Message):
    __slots__ = ["record_count", "reconciling"]
    RECORD_COUNT_FIELD_NUMBER: _ClassVar[int]
    RECONCILING_FIELD_NUMBER: _ClassVar[int]
    record_count: int
    reconciling: bool
    def __init__(self, record_count: _Optional[int] = ..., reconciling: bool = ...) -> None: ...

class GetZoneRequest(_message.Message):
    __slots__ = ["id", "resource_version"]
    ID_FIELD_NUMBER: _ClassVar[int]
    RESOURCE_VERSION_FIELD_NUMBER: _ClassVar[int]
    id: str
    resource_version: int
    def __init__(self, id: _Optional[str] = ..., resource_version: _Optional[int] = ...) -> None: ...

class ListZonesRequest(_message.Message):
    __slots__ = ["parent_id", "page_size", "page_token"]
    PARENT_ID_FIELD_NUMBER: _ClassVar[int]
    PAGE_SIZE_FIELD_NUMBER: _ClassVar[int]
    PAGE_TOKEN_FIELD_NUMBER: _ClassVar[int]
    parent_id: str
    page_size: int
    page_token: str
    def __init__(self, parent_id: _Optional[str] = ..., page_size: _Optional[int] = ..., page_token: _Optional[str] = ...) -> None: ...

class ListZonesResponse(_message.Message):
    __slots__ = ["items", "next_page_token"]
    ITEMS_FIELD_NUMBER: _ClassVar[int]
    NEXT_PAGE_TOKEN_FIELD_NUMBER: _ClassVar[int]
    items: _containers.RepeatedCompositeFieldContainer[Zone]
    next_page_token: str
    def __init__(self, items: _Optional[_Iterable[_Union[Zone, _Mapping]]] = ..., next_page_token: _Optional[str] = ...) -> None: ...



from google.protobuf import descriptor_pb2 as _descriptor_pb2
from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Optional as _Optional

DESCRIPTOR: _descriptor.FileDescriptor

class ResourceBehavior(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = []
    RESOURCE_BEHAVIOR_UNSPECIFIED: _ClassVar[ResourceBehavior]
    MOVABLE: _ClassVar[ResourceBehavior]
    UNNAMED: _ClassVar[ResourceBehavior]
    IMMUTABLE_NAME: _ClassVar[ResourceBehavior]

class FieldBehavior(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = []
    FIELD_BEHAVIOR_UNSPECIFIED: _ClassVar[FieldBehavior]
    IMMUTABLE: _ClassVar[FieldBehavior]
    IDENTIFIER: _ClassVar[FieldBehavior]
    INPUT_ONLY: _ClassVar[FieldBehavior]
    OUTPUT_ONLY: _ClassVar[FieldBehavior]
    MEANINGFUL_EMPTY_VALUE: _ClassVar[FieldBehavior]
    NON_EMPTY_DEFAULT: _ClassVar[FieldBehavior]
RESOURCE_BEHAVIOR_UNSPECIFIED: ResourceBehavior
MOVABLE: ResourceBehavior
UNNAMED: ResourceBehavior
IMMUTABLE_NAME: ResourceBehavior
FIELD_BEHAVIOR_UNSPECIFIED: FieldBehavior
IMMUTABLE: FieldBehavior
IDENTIFIER: FieldBehavior
INPUT_ONLY: FieldBehavior
OUTPUT_ONLY: FieldBehavior
MEANINGFUL_EMPTY_VALUE: FieldBehavior
NON_EMPTY_DEFAULT: FieldBehavior
FILE_DEPRECATION_DETAILS_FIELD_NUMBER: _ClassVar[int]
file_deprecation_details: _descriptor.FieldDescriptor
API_SERVICE_NAME_FIELD_NUMBER: _ClassVar[int]
api_service_name: _descriptor.FieldDescriptor
SERVICE_DEPRECATION_DETAILS_FIELD_NUMBER: _ClassVar[int]
service_deprecation_details: _descriptor.FieldDescriptor
SERVICE_PY_SDK_FIELD_NUMBER: _ClassVar[int]
service_py_sdk: _descriptor.FieldDescriptor
METHOD_DEPRECATION_DETAILS_FIELD_NUMBER: _ClassVar[int]
method_deprecation_details: _descriptor.FieldDescriptor
METHOD_PY_SDK_FIELD_NUMBER: _ClassVar[int]
method_py_sdk: _descriptor.FieldDescriptor
RESOURCE_BEHAVIOR_FIELD_NUMBER: _ClassVar[int]
resource_behavior: _descriptor.FieldDescriptor
MESSAGE_DEPRECATION_DETAILS_FIELD_NUMBER: _ClassVar[int]
message_deprecation_details: _descriptor.FieldDescriptor
MESSAGE_PY_SDK_FIELD_NUMBER: _ClassVar[int]
message_py_sdk: _descriptor.FieldDescriptor
FIELD_BEHAVIOR_FIELD_NUMBER: _ClassVar[int]
field_behavior: _descriptor.FieldDescriptor
SENSITIVE_FIELD_NUMBER: _ClassVar[int]
sensitive: _descriptor.FieldDescriptor
CREDENTIALS_FIELD_NUMBER: _ClassVar[int]
credentials: _descriptor.FieldDescriptor
FIELD_DEPRECATION_DETAILS_FIELD_NUMBER: _ClassVar[int]
field_deprecation_details: _descriptor.FieldDescriptor
FIELD_PY_SDK_FIELD_NUMBER: _ClassVar[int]
field_py_sdk: _descriptor.FieldDescriptor
ONEOF_BEHAVIOR_FIELD_NUMBER: _ClassVar[int]
oneof_behavior: _descriptor.FieldDescriptor
ONEOF_PY_SDK_FIELD_NUMBER: _ClassVar[int]
oneof_py_sdk: _descriptor.FieldDescriptor
ENUM_PY_SDK_FIELD_NUMBER: _ClassVar[int]
enum_py_sdk: _descriptor.FieldDescriptor
ENUM_DEPRECATION_DETAILS_FIELD_NUMBER: _ClassVar[int]
enum_deprecation_details: _descriptor.FieldDescriptor
ENUM_VALUE_DEPRECATION_DETAILS_FIELD_NUMBER: _ClassVar[int]
enum_value_deprecation_details: _descriptor.FieldDescriptor
ENUM_VALUE_PY_SDK_FIELD_NUMBER: _ClassVar[int]
enum_value_py_sdk: _descriptor.FieldDescriptor

class ServicePySDKSettings(_message.Message):
    __slots__ = ["name"]
    NAME_FIELD_NUMBER: _ClassVar[int]
    name: str
    def __init__(self, name: _Optional[str] = ...) -> None: ...

class MethodPySDKSettings(_message.Message):
    __slots__ = ["name"]
    NAME_FIELD_NUMBER: _ClassVar[int]
    name: str
    def __init__(self, name: _Optional[str] = ...) -> None: ...

class FieldPySDKSettings(_message.Message):
    __slots__ = ["name"]
    NAME_FIELD_NUMBER: _ClassVar[int]
    name: str
    def __init__(self, name: _Optional[str] = ...) -> None: ...

class MessagePySDKSettings(_message.Message):
    __slots__ = ["name"]
    NAME_FIELD_NUMBER: _ClassVar[int]
    name: str
    def __init__(self, name: _Optional[str] = ...) -> None: ...

class OneofPySDKSettings(_message.Message):
    __slots__ = ["name"]
    NAME_FIELD_NUMBER: _ClassVar[int]
    name: str
    def __init__(self, name: _Optional[str] = ...) -> None: ...

class EnumPySDKSettings(_message.Message):
    __slots__ = ["name"]
    NAME_FIELD_NUMBER: _ClassVar[int]
    name: str
    def __init__(self, name: _Optional[str] = ...) -> None: ...

class EnumValuePySDKSettings(_message.Message):
    __slots__ = ["name"]
    NAME_FIELD_NUMBER: _ClassVar[int]
    name: str
    def __init__(self, name: _Optional[str] = ...) -> None: ...

class DeprecationDetails(_message.Message):
    __slots__ = ["effective_at", "description"]
    EFFECTIVE_AT_FIELD_NUMBER: _ClassVar[int]
    DESCRIPTION_FIELD_NUMBER: _ClassVar[int]
    effective_at: str
    description: str
    def __init__(self, effective_at: _Optional[str] = ..., description: _Optional[str] = ...) -> None: ...



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: nebius/iam/v1/invitation.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from nebius.api.buf.validate import validate_pb2 as buf_dot_validate_dot_validate__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from nebius.api.nebius import annotations_pb2 as nebius_dot_annotations__pb2
from nebius.api.nebius.common.v1 import metadata_pb2 as nebius_dot_common_dot_v1_dot_metadata__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1enebius/iam/v1/invitation.proto\x12\rnebius.iam.v1\x1a\x1b\x62uf/validate/validate.proto\x1a\x1fgoogle/protobuf/timestamp.proto\x1a\x18nebius/annotations.proto\x1a\x1fnebius/common/v1/metadata.proto\"\xd4\x01\n\nInvitation\x12\x46\n\x08metadata\x18\x01 \x01(\x0b\x32\".nebius.common.v1.ResourceMetadataB\x06\xbaH\x03\xc8\x01\x01R\x08metadata\x12\x39\n\x04spec\x18\x02 \x01(\x0b\x32\x1d.nebius.iam.v1.InvitationSpecB\x06\xbaH\x03\xc8\x01\x01R\x04spec\x12=\n\x06status\x18\x03 \x01(\x0b\x32\x1f.nebius.iam.v1.InvitationStatusB\x04\xbaJ\x01\x05R\x06status:\x04\xbaJ\x01\x02\"Z\n\x0eInvitationSpec\x12 \n\x0b\x64\x65scription\x18\x01 \x01(\tR\x0b\x64\x65scription\x12\x1b\n\x05\x65mail\x18\x0b \x01(\tB\x03\xc0J\x01H\x00R\x05\x65mailB\t\n\x07\x63ontact\"\x9c\x02\n\x10InvitationStatus\x12\x33\n\x16tenant_user_account_id\x18\x01 \x01(\tR\x13tenantUserAccountId\x12\x39\n\nexpires_at\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\texpiresAt\x12;\n\x05state\x18\x03 \x01(\x0e\x32%.nebius.iam.v1.InvitationStatus.StateR\x05state\"[\n\x05State\x12\x0f\n\x0bUNSPECIFIED\x10\x00\x12\x0c\n\x08\x43REATING\x10\x04\x12\x0b\n\x07\x43REATED\x10\x05\x12\x0b\n\x07PENDING\x10\x01\x12\x0b\n\x07\x45XPIRED\x10\x02\x12\x0c\n\x08\x41\x43\x43\x45PTED\x10\x03\x42V\n\x14\x61i.nebius.pub.iam.v1B\x0fInvitationProtoP\x01Z+github.com/nebius/gosdk/proto/nebius/iam/v1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'nebius.iam.v1.invitation_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\024ai.nebius.pub.iam.v1B\017InvitationProtoP\001Z+github.com/nebius/gosdk/proto/nebius/iam/v1'
  _INVITATION.fields_by_name['metadata']._options = None
  _INVITATION.fields_by_name['metadata']._serialized_options = b'\272H\003\310\001\001'
  _INVITATION.fields_by_name['spec']._options = None
  _INVITATION.fields_by_name['spec']._serialized_options = b'\272H\003\310\001\001'
  _INVITATION.fields_by_name['status']._options = None
  _INVITATION.fields_by_name['status']._serialized_options = b'\272J\001\005'
  _INVITATION._options = None
  _INVITATION._serialized_options = b'\272J\001\002'
  _INVITATIONSPEC.fields_by_name['email']._options = None
  _INVITATIONSPEC.fields_by_name['email']._serialized_options = b'\300J\001'
  _globals['_INVITATION']._serialized_start=171
  _globals['_INVITATION']._serialized_end=383
  _globals['_INVITATIONSPEC']._serialized_start=385
  _globals['_INVITATIONSPEC']._serialized_end=475
  _globals['_INVITATIONSTATUS']._serialized_start=478
  _globals['_INVITATIONSTATUS']._serialized_end=762
  _globals['_INVITATIONSTATUS_STATE']._serialized_start=671
  _globals['_INVITATIONSTATUS_STATE']._serialized_end=762
# @@protoc_insertion_point(module_scope)



from nebius.api.nebius import annotations_pb2 as _annotations_pb2
from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class UserAccountExternalId(_message.Message):
    __slots__ = ["federation_user_account_id", "federation_id"]
    FEDERATION_USER_ACCOUNT_ID_FIELD_NUMBER: _ClassVar[int]
    FEDERATION_ID_FIELD_NUMBER: _ClassVar[int]
    federation_user_account_id: str
    federation_id: str
    def __init__(self, federation_user_account_id: _Optional[str] = ..., federation_id: _Optional[str] = ...) -> None: ...

class UserAccountStatus(_message.Message):
    __slots__ = ["state"]
    class State(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
        __slots__ = []
        STATE_UNSPECIFIED: _ClassVar[UserAccountStatus.State]
        ACTIVE: _ClassVar[UserAccountStatus.State]
        INACTIVE: _ClassVar[UserAccountStatus.State]
        DELETING: _ClassVar[UserAccountStatus.State]
    STATE_UNSPECIFIED: UserAccountStatus.State
    ACTIVE: UserAccountStatus.State
    INACTIVE: UserAccountStatus.State
    DELETING: UserAccountStatus.State
    STATE_FIELD_NUMBER: _ClassVar[int]
    state: UserAccountStatus.State
    def __init__(self, state: _Optional[_Union[UserAccountStatus.State, str]] = ...) -> None: ...



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: nebius/audit/v2/request.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import struct_pb2 as google_dot_protobuf_dot_struct__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1dnebius/audit/v2/request.proto\x12\x0fnebius.audit.v2\x1a\x1cgoogle/protobuf/struct.proto\"\x88\x02\n\x07Request\x12\x1b\n\tclient_ip\x18\x01 \x01(\tR\x08\x63lientIp\x12\x1d\n\nuser_agent\x18\x02 \x01(\tR\tuserAgent\x12\x1d\n\nrequest_id\x18\x03 \x01(\tR\trequestId\x12\x37\n\nparameters\x18\x04 \x01(\x0b\x32\x17.google.protobuf.StructR\nparameters\x12%\n\x0eidempotency_id\x18\x05 \x01(\tR\ridempotencyId\x12\x19\n\x08trace_id\x18\x06 \x01(\tR\x07traceId\x12\'\n\x0fja3_fingerprint\x18\x07 \x01(\tR\x0eja3FingerprintBW\n\x16\x61i.nebius.pub.audit.v2B\x0cRequestProtoP\x01Z-github.com/nebius/gosdk/proto/nebius/audit/v2b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'nebius.audit.v2.request_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\026ai.nebius.pub.audit.v2B\014RequestProtoP\001Z-github.com/nebius/gosdk/proto/nebius/audit/v2'
  _globals['_REQUEST']._serialized_start=81
  _globals['_REQUEST']._serialized_end=345
# @@protoc_insertion_point(module_scope)
